/*
  COPYRIGHT (C) 2003 APPLIED NEUROINFORMATIC GROUP - UNIVERSITY OF BIELEFELD.

  ALL RIGHTS RESERVED.

  REDISTRIBUTION AND USE IN SOURCE AND BINARY FORM, WITH OR WITHOUT
  MODIFICATION, REQUIRE THE PERMISSION OF THE COPYRIGHT HOLDERS.

  COMMERCIAL USE WITHOUT THE EXPLICIT PERMISSION OF THE COPYRIGHT HOLDERS
  IS FORBIDDEN

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/



#ifndef CKERNELCACHE_H
#define CKERNELCACHE_H
#include "CKernel.h"
#include "CDenseVector.h"
#include "CSVCandidate.h"
#include "CDataset.h"
#include "Macros.h"
#include <cmath>
#include <list>
#include <vector>


template<class Type>
class CKernelCache : public CObject<Type>{
	struct TCachedItem {
		int          iIndex;
		TCachedItem* ptPreItem;
		TCachedItem* ptSucItem;
		CDenseVector<Type>* ptKernelVector;
	};


protected:
	TCachedItem** pptCachedItems;
	TCachedItem*  ptStorageHead;
	TCachedItem*  ptStorageEnd;

	Type tCacheSize;
	int iMaxRows;
	int iNumCachedRows;
	int iNumItemsToUse;
	Type tDTPos, tDTNeg;
	bool bFastGaussian;

public:
	vector< CSVCandidate<Type> >* ptDataset;
	CKernel<Type> *ptKernel;
public:
	CKernelCache( CKernel<Type>* pptKernel,
	              vector< CSVCandidate<Type> > *ptDataset,
	              Type tDTPos = 0, Type tDTNeg = 0,
	              Type tMaxMegaByte = 10);
	~CKernelCache();

	static void threadFunction(void* pvData);
	const CDenseVector<Type>* requestRow(int i1, vector<int>* ptActiveCols = NULL);
	void init();
	void clear();
	void calculateRow(CDenseVector<Type>* ptRow, int iIndex);
	/*!
	 * Update pointer to kernel function object.
	 * @param ptKernel New pointer
	 **/
	void setKernel(CKernel<Type>* ptKernel);
	/**
	   Clear current cache. If iSubsetSize is unequal to -1 only the first iSubsetSize elements of the
	   dataset will be cached. In this case, a iSubsetSize X iSubsetSize matrix will similated.
	   @param Size of subset to cache. Whole dataset if equal to -1
	**/
	void reorganize(int iSubsetSize=-1);

	/**
	   Returns percentage of cached rows
	   @return Percentage of rows currently cached
	**/
	float fillState();
	/**
	   Return left cache meory in MB
	   @return Memory in MB
	**/
	float freeSpace();

	/**
	   Set maximal cache size in MB
	   @param Cache size
	**/
	void setCacheSize(Type tSize) {tCacheSize=tSize;};

	virtual string className() const { return "CKernelCache";};
	virtual bool isA(const char* acClass) const { if(string(acClass) == this->className()) return true; else return CObject<Type>::isA(acClass);};

	void setDiagonalTerm(Type tDT);
	void setDiagonalTerm(Type tDTPos, Type tDTNeg);
};


#endif
