/*
  COPYRIGHT (C) 2003 APPLIED NEUROINFORMATIC GROUP - UNIVERSITY OF BIELEFELD.

  ALL RIGHTS RESERVED.

  REDISTRIBUTION AND USE IN SOURCE AND BINARY FORM, WITH OR WITHOUT
  MODIFICATION, REQUIRE THE PERMISSION OF THE COPYRIGHT HOLDERS.

  COMMERCIAL USE WITHOUT THE EXPLICIT PERMISSION OF THE COPYRIGHT HOLDERS
  IS FORBIDDEN

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/



#ifndef COBJECT_H
#define COBJECT_H

#include <fstream>
#include <iostream>
#include <string>
#include <stdio.h>

class CArchiv;

using namespace std;
enum VERBOSE_LEVEL { OFF, STATUS, ERROR, VERBOSE};
enum IO_MODE { READ, WRITE};
enum DATATYPE { INT, FLOAT, DOUBLE };

#define COUT(msg,level) { if( level && level <= (int) this->verbose() ) cout << msg << endl; }




/*!
 * @brief Base class for all object.
 *
 **/
template<class Type>
class CObject {
protected:
	unsigned char ucVerbose;

public:
	//! Constructor
	CObject();

	//! Destructor
	virtual ~CObject();


	//! Read/write from binary stream.
	/*! The functions handles different data types e.g for reading float objects
	 * in a double instance, etc.
	 * @param fstream Reference to binary stream
	 * @param mode Switches between reading and writing
	 * @return True if successful, false otherwise
	 **/
	virtual bool serialize(fstream& stream, IO_MODE mode=READ);
	virtual bool serialize2(CArchiv& tArchiv) {return true;};

	//! Returns the class name
	virtual string className() const  { return string("CObject"); };

	//! Set the verbose level
	/*!
	 * @param tVerbose Level, OFF=0, STATUS, ERROR, DEBUG;
	 **/
	void setVerbose(VERBOSE_LEVEL tVerbose) {this->ucVerbose= (unsigned char) tVerbose;}

	//! Return current verbose level
	/*!
	 * @return Verbose Level
	 **/
	VERBOSE_LEVEL verbose(void) const { return (VERBOSE_LEVEL) ucVerbose;}

	//! Check if the object is an instance of the class with given name.
	/*
	 * If the object class doesn't match the given class, the method searches upward in the class hierarchy
	 * @param acClass Name of class
	 */
	virtual bool isA(const char* acClass) const { if(string(acClass) == string(this->className())) return true; else return false; };

	//! Returns the template data type
	/*!
	 * @return INT/FLOAT/DOUBLE
	 **/
	DATATYPE dataType() const ;
};

#endif
