/*
  COPYRIGHT (C) 2003 APPLIED NEUROINFORMATIC GROUP - UNIVERSITY OF BIELEFELD.

  ALL RIGHTS RESERVED.

  REDISTRIBUTION AND USE IN SOURCE AND BINARY FORM, WITH OR WITHOUT
  MODIFICATION, REQUIRE THE PERMISSION OF THE COPYRIGHT HOLDERS.

  COMMERCIAL USE WITHOUT THE EXPLICIT PERMISSION OF THE COPYRIGHT HOLDERS
  IS FORBIDDEN

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef COBJECTFACTORY_H
#define COBJECTFACTORY_H


#include "CObject.h"
#include <string>

using namespace std;

extern "C"{
	typedef void* _create_double_object();
	typedef void* _create_float_object();
}


/*!
 * @brief Produces objects of given name and type
 *
 * The object factory can produce any object for which a corresponding
 * '*Factory.so' shared object is located in the library path. The object
 * to produce is specified by the name and by the datatype. The main area
 * of application is the serialization of certain objects.
 *
 **/


class CObjectFactory {
private:
	string strCurrent;
	void* hObject;
	static CObjectFactory* pinstance;
public:
	~CObjectFactory();
	static CObjectFactory* instance();
	/*!
	 * Produce object of given name and type
	 * @param acObjectName Name of class
	 * @param tType Template type
	 * @return Pointer to object instance of NULL
	 **/
	void* produceObject(const char* acObjectName, DATATYPE tType);

	/*!
	 * Produce object of given name and type
	 * @param acObjectName Name of class
	 * @param tType Template type
	 * @return Pointer to object instance of NULL
	 **/
	void* produceObject(const string strObjectName, DATATYPE tType);
	bool canProduce(const char* acObjectName);
	bool canProduce(const string strObjectName);
protected:
	CObjectFactory();
};

#endif
