/*
  COPYRIGHT (C) 2003 APPLIED NEUROINFORMATIC GROUP - UNIVERSITY OF BIELEFELD.

  ALL RIGHTS RESERVED.

  REDISTRIBUTION AND USE IN SOURCE AND BINARY FORM, WITH OR WITHOUT
  MODIFICATION, REQUIRE THE PERMISSION OF THE COPYRIGHT HOLDERS.

  COMMERCIAL USE WITHOUT THE EXPLICIT PERMISSION OF THE COPYRIGHT HOLDERS
  IS FORBIDDEN

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/



#ifndef CONECLASSSVM_H
#define CONECLASSSVM_H

#include "CClassifier.h"
#include "CKernel.h"
#include "CSVM.h"
#include "CDataset.h"
#include "CVector.h"
#include <vector>

/*!
 * @brief OneClass SVM algorithm for estimating the support of a data distribution.
 *
 * dfdf
 * @author Thorsten Twellmann (ttwellma@TechFak.Uni-Bielefeld.de
 */
template<class Type>
class COneClassSVM : public CClassifier<Type>
{
protected:
	double tNu;
	double tRho;
	double tKKTEpsilon;
	double tBound;
	int iNumIteration;

	CKernel<Type>* ptKernel;
	CKernelCache<Type>* ptCache;
	vector<CSVCandidate<Type> > tWorkingSet;


public:
	COneClassSVM(CKernel<Type>* ptKernel = new CGaussianKernel<Type>(), int iCacheSize = 100);
	~COneClassSVM();

	int classify(const CVector<Type>& tPattern) const;
	int classify(Type* ptPattern, Type& rtConfidence) const;
	void classify(Type* ptPattern, Type& rtClass, Type& rtConfidence) const;
	void classify(const CVector<Type>& rtPattern, Type& rtClass, Type& rtConfidence) const;
	void init(const CDataset<Type>& tTrainSet);
	void train(const CDataset<Type>& tTrainSet);
	CDataset<Type> getSupportData();
	void setKernel(CKernel<Type>* ptKernel);
	const CKernel<Type>* kernel() {
		return ptKernel;
	};

	/*! Set new cache size (in MByte)
	 * @param tSize Cache size
	 **/
	void setCacheSize (Type tSize) {
		ptCache->setCacheSize(tSize);
	};


	void setRegularization(Type tNu) { this->tNu = tNu;};
	void setEpsilon(Type tEpsilon) { tKKTEpsilon = tEpsilon;};
protected:
	int takeStep(int a, int b);
	int examineExample(int i);
	int findBestPartner(int iIndex);
	bool checkKKT(int iIndex);
	Type calculateOutput(int iIndex);

};

#endif
