/*
  COPYRIGHT (C) 2003 APPLIED NEUROINFORMATIC GROUP - UNIVERSITY OF BIELEFELD.

  ALL RIGHTS RESERVED.

  REDISTRIBUTION AND USE IN SOURCE AND BINARY FORM, WITH OR WITHOUT
  MODIFICATION, REQUIRE THE PERMISSION OF THE COPYRIGHT HOLDERS.

  COMMERCIAL USE WITHOUT THE EXPLICIT PERMISSION OF THE COPYRIGHT HOLDERS
  IS FORBIDDEN

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/



#ifndef COPTIMIZER_H
#define COPTIMIZER_H

#include <CMatrix.h>
#include <CDenseVector.h>
#include <CObject.h>

/*! @brief Base class for optimisation techniques;
  ...
**/

template<class Type>
class COptimizer : public CObject<Type> {
private:
	bool m_bMaximize;
	CDenseVector<Type> m_tPosition;
	Type m_tSenseSign;
public:
	COptimizer();
	string className() const  { return string("COptimizer");};
	bool isA(const char* acClass) const;
	virtual ~COptimizer();
	virtual Type optimize() = 0;
	//! Set optimisation sense
	/*! Specifes whether the function should be maximized or minimized
	 * @param bValue If true function will be maximized
	 **/
	void setMaximize( bool bValue );


	//! Set initial parameterisation
	/*! Set initial position in the parameter space
	 * @param tPos Initial position
	 **/
	void setPosition(CDenseVector<Type> tPos);

	//! Return current parameterisation
	CDenseVector<Type> getPosition() const;

	CDenseVector<Type>& position();



	Type sign() const { return m_tSenseSign;};

	bool isMaximize() const { return m_bMaximize;};

	virtual Type functionValueAt(CDenseVector<Type>& tPos) = 0;

	virtual CDenseVector<Type> partialDerivativeAt(CDenseVector<Type>& tPos) = 0;
	bool serialize2(CArchiv& tA);
};
#endif
