/*
  COPYRIGHT (C) 2003 APPLIED NEUROINFORMATIC GROUP - UNIVERSITY OF BIELEFELD.

  ALL RIGHTS RESERVED.

  REDISTRIBUTION AND USE IN SOURCE AND BINARY FORM, WITH OR WITHOUT
  MODIFICATION, REQUIRE THE PERMISSION OF THE COPYRIGHT HOLDERS.

  COMMERCIAL USE WITHOUT THE EXPLICIT PERMISSION OF THE COPYRIGHT HOLDERS
  IS FORBIDDEN

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/



#ifndef CPROJECTION_H
#define CPROJECTION_H


#include "CDataset.h"
#include "CDenseVector.h"
#include "CObject.h"

/*!
 * @brief Base class for projection algorithms like PCA, ICA, KPCA, etc
 **/
template<class Type>
class CProjection : public CObject<Type>
{
protected:
	int iNumDirections;       /**< Maximal number of available directions */

public:
	CProjection(){};

	virtual ~CProjection(){};

	virtual void adapt(const CDataset<Type>& tDataset, int iNumDirections)=0;
	virtual CDataset<Type> project(CDataset<Type>& rtDs, int iNumDirections=0)=0;
	virtual CDenseVector<Type> project(const CDenseVector<Type>& rtVector, int iNumDirections=0)=0;
	virtual string className() const{ return string("CProjection");};

	/*!
	 * Returns maximal number of possible projection direction
	 * @return Max. number of directions
	 **/
	int directions() { return iNumDirections;};


};

#endif
