#ifndef CSVMLIGHT_H
#define CSVMLIGHT_H


/*
  COPYRIGHT (C) 2003 APPLIED NEUROINFORMATIC GROUP - UNIVERSITY OF BIELEFELD.

  ALL RIGHTS RESERVED.

  REDISTRIBUTION AND USE IN SOURCE AND BINARY FORM, WITH OR WITHOUT
  MODIFICATION, REQUIRE THE PERMISSION OF THE COPYRIGHT HOLDERS.

  COMMERCIAL USE WITHOUT THE EXPLICIT PERMISSION OF THE COPYRIGHT HOLDERS
  IS FORBIDDEN

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include "CClassifier.h"
#include "CSVCandidate.h"
#include "CKernel.h"
#include "CArchiv.h"
#include "CSVMModel.h"

/*! @short Wrapper for SVMLight library written by T. Joachims
  ....
**/
template<class Type>
class CSVMLight : public CClassifier<Type> {
protected:
	vector< CSVCandidate<Type> > m_tSupportVectors;
	Type m_tBias;
	Type m_tC;
	Type m_tEpsilon;
	Type m_tPositiveCost;
	Type m_tSigmoidA;
	Type m_tSigmoidB;
	int m_iCacheSize;
	int m_iMaxQPSize;
	int m_iCalibrationFolds;
	bool m_bCalibrated;
	bool m_bLinearCached;
	CKernel<Type>* m_ptKernel;
	CDenseVector<Type> m_tWeight;

public:
	//! Constructor
	CSVMLight();

	//! Destructor
	~CSVMLight();


	void train(const CDataset<Type>& rtDs);


	void classify(const CVector<Type>&, Type&, Type&) const;


	//! Set kernel object
	void setKernel(CKernel<Type>* ptKernel);

	//! Set size if cache in MByte
	void setCacheSize(int iSize);

	//! Set maximal size of quadratic program (>=2)
	void setMaxQPSize(int iSize);

	//! Set regularization value
	void setRegularization(Type tC);

	//! Set number of calibration folds
	void setCalibrationFolds(int iFolds);

	//! Set maximal termination error
	void setMaxTerminationError(Type tError);

	//! Set cost factor, by which error on postive items outweight errors for negative items
	void setCostPositiveFactor(Type tCost);


	//! Get model of trained svm
	CSVMModel<Type> getModel() const;


	//! Activate output calibration
	void setCalibratedOutput(bool bSwitch);

	const CKernel<Type>* kernel() {
		return m_ptKernel;
	};

	bool serialize2(CArchiv&);

	int classify(const CVector<Type>&) const;


	string className() const { return string("CSVMLight");};


	bool isA(const char* acClass) const { if(string(acClass) == "CSVMLight") return true; else return CClassifier<Type>::isA(acClass);};

protected:
	void solveQP(const CDataset<Type>& rtDs);
	bool calibrate(CDenseVector<double> tIsPositive, CDenseVector<double> tConfidence, Type& tSigmoidA, Type& tSigmoidB);

};


#endif
