/*
  COPYRIGHT (C) 2003 APPLIED NEUROINFORMATIC GROUP - UNIVERSITY OF BIELEFELD.

  ALL RIGHTS RESERVED.

  REDISTRIBUTION AND USE IN SOURCE AND BINARY FORM, WITH OR WITHOUT
  MODIFICATION, REQUIRE THE PERMISSION OF THE COPYRIGHT HOLDERS.

  COMMERCIAL USE WITHOUT THE EXPLICIT PERMISSION OF THE COPYRIGHT HOLDERS
  IS FORBIDDEN

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/



#ifndef __CVECTOROPERATIONS_H
#define __CVECTOROPERATIONS_H

template<typename T> class CVector;
template<typename T> class CSparseVector;
template<typename T> class CDenseVector;

// forward declaring string doesn't seem to work the usual way
#include <string>

template<typename T>
class CVectorOperations
{
public:
	typedef T (*ScalarFunction)(const CVector<T>& a, const CVector<T>& b);
	typedef void (*InplaceFunction)(CVector<T>& a, const CVector<T>& b);
	typedef void (*InplaceFunctionArg)(CVector<T>&, const CVector<T>&, float, float);
	typedef void (*CopyFunction)(CDenseVector<T>&, const CVector<T>&, const CVector<T>&); // CDenseVector output for now; maybe other types later on

	typedef CSparseVector<T> sparse;
	typedef CDenseVector<T> dense;

	// methods to retrieve specific operations
	InplaceFunction getElementProduct(const CVector<T>&, const CVector<T>&) const;
	InplaceFunction getElementDivision(const CVector<T>&, const CVector<T>&) const;
	InplaceFunctionArg getElementAdapt(const CVector<T>&, const CVector<T>&) const;
	ScalarFunction getScalarProduct(const CVector<T>&, const CVector<T>&) const;
	ScalarFunction getCommonProduct(const CVector<T>&, const CVector<T>&) const;
	ScalarFunction getEuclideanDistance(const CVector<T>&, const CVector<T>&) const;
	ScalarFunction getCoCosineDistance(const CVector<T>&, const CVector<T>&) const;
	InplaceFunction getElementCopy(const CVector<T>&, const CVector<T>&) const;
	CopyFunction getElementAddition(const CVector<T>&, const CVector<T>&) const;
	CopyFunction getElementSubtraction(const CVector<T>&, const CVector<T>&) const;


	/// get Singleton instance
	static CVectorOperations<T>& getInstance();

protected:
	int getIndex(const std::type_info&, const std::type_info&) const;
	ScalarFunction lookupScalarFunction(const ScalarFunction*, const CVector<T>&, const CVector<T>&) const;
	InplaceFunction lookupInplaceFunction(const InplaceFunction*, const CVector<T>&, const CVector<T>&) const;
	InplaceFunctionArg lookupInplaceFunctionArg(const InplaceFunctionArg*, const CVector<T>&, const CVector<T>&) const;
	CopyFunction lookupCopyFunction(const CopyFunction*, const CVector<T>&, const CVector<T>&) const;

	CVectorOperations();
	CVectorOperations(const CVectorOperations&);

private:
	ScalarFunction scalarProducts[4];
	ScalarFunction euclideanDistances[4];
	ScalarFunction coCosineDistances[4];
	InplaceFunction elementProducts[4];
	InplaceFunction elementDivisions[4];
	InplaceFunction elementCopies[4];
	CopyFunction elementAdditions[4];
	CopyFunction elementSubtractions[4];
	InplaceFunctionArg elementAdapts[4];
};

/*
 * Local variables:
 * mode: c++
 * End:
 */
#endif
