#ifndef MACROS_H
#define MACROS_H
#include <ctime>
#include <cstdlib>
#include <cmath>
#include <vector>
#include <stdio.h>
#include <iostream>
#include <CObject.h>
using namespace std;
#ifndef INFINITY
#define INFINITY HUGE_VAL
#endif

#define MSG(x) {cout << x << endl;}
#define ABS(x)   ( ((x) < 0)   ? -(x) : (x))
#define SIGN(x)  ( ((x) < 0)   ? -1   : +1)

#define MAX(x,y) ( ((x) > (y)) ? (x)  : (y))
#define MIN(x,y) ( ((x) < (y)) ? (x)  : (y))

// template<class Type>
// Type MAX(Type x, Type y)
// {
//    if(x>=y)
//       return x;
//    else
//       return y;
// };


// template<class Type>
// Type MIN(Type x, Type y)
// {
//    if(x<y) return x;
//    else return y;
// };

// template<class Type>
// Type SIGN(Type x)
// {
//    if(x>0) return (Type)+1;
//    else return (Type)-1;
// };

// template<class Type>
// Type ABS(Type x)
// {
//    if(x>0) return x;
//    else return (-x);
// };







template<class Type>
void readValueOfType(Type& rtValue, fstream& stream, DATATYPE tType)
{
	if(tType == INT) {
		int iTmp;
		stream.read((char*) &iTmp, sizeof(int));
		rtValue = (Type) iTmp;
	} else if(tType == FLOAT) {
		float fTmp;
		stream.read((char*) &fTmp, sizeof(float));
		rtValue = (Type) fTmp;
	} else if(tType == DOUBLE) {
		double dTmp;
		stream.read((char*) &dTmp, sizeof(double));
		rtValue = (Type) dTmp;
	} else return;
}

/*
  return random (integer) number in [x,..,y]
*/
#define IRAND(x,y) ((x) + (int)(((float) (y - x)) * rand()/(RAND_MAX+1.0)))

/**
   returns value of element with rank iRank in the given array.
   NOTE: The sorting of the array values will be changed!
**/
template<class Type>
Type getRank(int iRank, int iDim, Type* pfVector);

/******************************************************************************/
/**
   Macros for error handling
**/

/*-NAN checks----------------------------------------------------------------*/

#define ML_NAN_CHK(var)  if(!isnan(var))  {cerr << "ML_NAN_CHK failed in File: " << __FILE__ << " Line: " << __LINE__ << ". " << #var << "!= NAN" << endl; abort();}

#define ML_NOT_NAN_CHK(var)  if(isnan(var))  {cerr << "ML_NOT_NAN_CHK failed in File: " << __FILE__ << " Line: " << __LINE__ << ". " << #var << "== NAN" << endl; abort();}
/*-NULL checks----------------------------------------------------------------*/

#define ML_NULL_CHK(var)  if((var) != NULL)  {cerr << "ML_NULL_CHK failed in File: " << __FILE__ << " Line: " << __LINE__ << ". " << #var << "!= NULL" << endl; abort();}

#define ML_NOT_NULL_CHK(var)  if((var) == NULL)  {cerr << "ML_NOT_NULL_CHK failed in File: " << __FILE__ << " Line: " << __LINE__ << ". " << #var << "== NULL" << endl; abort();}

/*-BOOL check-----------------------------------------------------------------*/

#define ML_BOOL_CHK(var)  if(!(var))  {cerr << "ML_BOOL_CHK failed in File: " << __FILE__ << " Line: " << __LINE__ << ". " << #var << "=" << var << endl; abort();}

/*-Less - Less equal----------------------------------------------------------*/

#define ML_LS_CHK(var, val)  if((var) >= (val))  {cerr << "ML_LS_CHK failed in File: " << __FILE__ << " Line: " << __LINE__ << ". " << #var << "=" << var << " >= " << val << endl; abort();}

#define ML_LEQ_CHK(var, val)  if((var) > (val))  {cerr << "ML_LEQ_CHK failed in File: " << __FILE__ << " Line: " << __LINE__ << ". " << #var << "=" << var << " > " << val << endl; abort();}

/*-Greater - Greater equal----------------------------------------------------*/

#define ML_GR_CHK(var, val)  if((var) <= (val))  {cerr << "ML_GR_CHK failed in File: " << __FILE__ << " Line: " << __LINE__ << ". " << #var << "=" << var << " <= " << val << endl; abort();}

#define ML_GEQ_CHK(var, val)  if((var) < (val))  {cerr << "ML_GEQ_CHK failed in File: " << __FILE__ << " Line: " << __LINE__ << ". " << #var << "=" << var << " < " << val << endl; abort();}

/*-Equal - Not equal----------------------------------------------------------*/

#define ML_EQ_CHK(var, val)  if((var) != (val))  {cerr << "ML_EQ_CHK failed in File: " << __FILE__ << " Line: " << __LINE__ << ". " << #var << "=" << var << " != " << val << endl; abort();}

#define ML_NEQ_CHK(var, val)  if((var) == (val))  {cerr << "ML_NEQ_CHK failed in File: " << __FILE__ << " Line: " << __LINE__ << ". " << #var << "=" << var << " == " << val << endl; abort();}

/*-Out of range---------------------------------------------------------------*/

#define ML_RANGE_CHK(var, lower, upper)  if((var)<(lower) && (var)>(upper))  {cerr << "ML_RANGE_CHK failed in File: " << __FILE__ << " Line: " << __LINE__ << ". " << #var << "=" << var << " out of range ["<<lower <<","<<upper<<"]" << endl; abort();}

/*- Warning to stderr - execution will continue ------------------------------*/
#define ML_WARNING_MSG(x) {cerr << "ML_WARNING_MSG in File: " << __FILE__ << " Line: " << __LINE__ << endl << x << endl << endl; cerr << endl;}


/*- Warning to stderr - program will exit ------------------------------------*/
#define ML_CRITICAL_MSG(x) {cerr << "ML_CRITICAL_MSG in File: " << __FILE__ << " Line: " << __LINE__ << endl << x << endl << endl; abort();}

/*- Message to cout ----------------------------------------------------------*/
#define ML_MSG(x) {cerr << "ML_MSG in File: " << __FILE__ << " Line: " << __LINE__ << endl << x << endl << endl; cerr << endl;}


/******************************************************************************/
/**
   Simulates nested for-loops.
   After a call with bInit=True, each function call increments rvtIndices according
   to an iteration of nested loops with a range from rvtStart to rtvEnd (including).
   The incrementation begins with rvtIndices[0].
   @param rvtStart   start values for all loops.
   @param rvtEnd     end values for all loops.
   @param rvtIndices values for the actual iteration.
   @param bInit      initializes the loop


**/

template<class Type>
void nDimFor(const vector<Type>& rvtStart, const vector<Type>& rvtEnd, vector<Type>& rvtIndices, bool bInit);

template<class Type>
void nDimForReverse(const vector<Type>& rvtStart, const vector<Type>& rvtEnd, vector<Type>& rvtIndices, bool bInit);



/******************************************************************************/
/**
   Math functions
**/

double gammln(double xx);
double factrl(int n);
double factln(int n);
double bico(int n,int k);
float gauss_rnd(void);
void init_rnd_seed();


#endif
