#ifndef __MYSQL_DATABASE_H
#define __MYSQL_DATABASE_H

#include "DatabaseException.h"
#include "CDataset.h"
#include "CSOM.h"

#include <mysql.h>

#define LOAD_LABEL          0x01
#define LOAD_KEYWORDS       0x02
#define LOAD_PROTOTYPES     0x04

template<class Type> class CSOM;

struct strEqual {
	bool operator()(const char* pc1, const char* pc2) const {
		return (strcmp(pc1, pc2) == 0);
	}
};


class CMysqlDatabaseException : public CDatabaseException
{
private:
	std::string mysql_msg;

public:
	CMysqlDatabaseException(const char* msg, MYSQL* conn, const char* file, int line) :
		CDatabaseException(msg, file, line), mysql_msg(mysql_error(conn)) {
	}

	virtual const std::string getMessage() const {
		return CDatabaseException::getMessage() + mysql_msg;
	}
};



class CMysqlDatabase
{
private:
	MYSQL  *pMysql;
	bool   sparse;
	string strDatabase;

public:
	CMysqlDatabase(const char* pcHost, const char* pcUser,
	               const char* pcPwd, const char* pcDb, int iPort = 0);

	~CMysqlDatabase();

	void fetchBagOfWords(CDataset<float>& rtDataset,
	                     const char* pcQuery, const char* pcTopic=NULL);

	void fetchBagOfWords(CDataset<float>& rtDataset,
	                     char* pcDictQuery, char* pcDocQuery, char* pcTopicTable=NULL,
	                     int iUseTfIdf=1);

	void fetchPyramidOfWords(CDataset<float>& rtDataset,
	                         int iNumLevels, int iLow, int iHigh,
	                         char *pcWhereDict=NULL, char *pcDocQuery=NULL,
	                         char *pcTopicTable=NULL,
	                         int  iUseTfIdf=1);

	void loadData(CDataset<float>& rtDataset,
	              char* pcTable, int iInp0, int iInp1, int iOut0, int iOut1);

	void saveSOM(CSOM<float>& rtSOM, const char* pcName, int iMappingOnly=0);
	void loadSOM(CSOM<float>& rtSOM, const char* pcName, int iVisible=3, int iMode=6);
	void loadFlow(CSOM<float>& rtSOM, const char* pcSomName, const char* pcFlowTable);
};


#endif
