extern "C" {
#include <nst.h>
#include <nst_strg.h>
#include <nst_dlfcn.h>
typedef struct nst_winfo_ nst_winfo_t;
}

void  setOut(unitptr, int f, int p, void* pNew);

class NstMethodUnit;
class NstUnit {
   friend class NstMethodUnit;
protected:
   NstUnit (const char* pcLabel, unitptr uHost);
   NstUnit (const char* pcLabel, int iField, ...);

public:
   const unitptr getUnit () const {return u;}
   virtual ~NstUnit ();

   /* Create a method sub unit for a class container.
      Typical usage is 
      NstMethodUnit uMethod;
      uMethod = createMethodUnit("name", ExecMethod<C,&C::method1>);
      uMethod.inp ("s2t20", "some pins");

      uMethod = createMethodUnit("other", ExecMethod<C,&C::method2>);

      The NstMethodUnit instance is useless thereafter. It's destructor
      doesn't destory the created NST unit (unlike NstUnit's destructor).
      So deleting NstMethodUnit is harmless.
   */
   NstMethodUnit createMethodUnit (const char* pcName, 
                                   int (*execMethod)(unitptr));

   void inp (const char* pcFormat, const char* pcDescription);
   void out (const char* pcFormat, const char* pcDescription);
   void inp (const char* pcDescription, ...);
   void out (const char* pcDescription, ...);

   void inpInfo (int f, const char* pcDescription);
   void outInfo (int f, const char* pcDescription);

   int numInputs () const { return nst_inputs_of(u); }
   int numOutputs () const { return nst_outputs_of(u); }

   float& fInp  (int f, int p) {return fNstInp (u,f,p);}
   float& fOut  (int f, int p) {return fNstOut (u,f,p);}

   void*& pvInp (int f, int p) {return pvNstInp (u,f,p);}
   void*& pvOut (int f, int p) {return pvNstOut (u,f,p);}
   void   setOut(int f, int p, void* pNew) {::setOut (u,f,p, pNew);}

   int*&  piInp (int f, int p) {return piNstInp (u,f,p);}
   int*&  piOut (int f, int p) {return piNstOut (u,f,p);}

   float*& pfInp (int f, int p) {return pfNstInp (u,f,p);}
   float*& pfOut (int f, int p) {return pfNstOut (u,f,p);}

   // byte vectors
   unsigned char*& pucInp (int f, int p) {return pcNstInp (u,f,p);}
   unsigned char*& pucOut (int f, int p) {return pcNstOut (u,f,p);}

   // text pins are not writeable as usual
   const char* pcInp (int f, int p) {return ptNstInp (u,f,p);}
   const char* pcOut (int f, int p) {return ptNstOut (u,f,p);}
   // use the following methods to write text to pins
   char*  strSet (int f, int p, const char* pcText) {
      return NstOutStrCpy (u,f,p, const_cast<char*>(pcText));
   }
   char*  strCat (int f, int p, const char* pcText) {
      return NstOutStrCat (u,f,p, const_cast<char*>(pcText));
   }

   pin* pInp(int f, int p) {return pNstInp (u,f,p);}
   pin* pOut(int f, int p) {return pNstOut (u,f,p);}

   virtual int exec  () = 0;
   virtual int adapt () {return OK;}
   virtual int ctrl  (int iMode) {return OK;}
   virtual int load  (FILE *fp);
   virtual int save  (FILE *fp);
   
private:
   NstUnit () : u(0) {};
   static int exec   (unitptr u);
   static int adapt  (unitptr u);
   static int ctrl   (int iMode, unitptr u);
   static int load   (FILE *fp, unitptr u);
   static int save   (FILE *fp, unitptr u);
   static int remove (unitptr u);
   
private:
   unitptr u; // attached NST unit
};

class NstMethodUnit : private NstUnit {
   friend class NstUnit;
public:
   NstMethodUnit () : NstUnit () {};
   ~NstMethodUnit ();

   using NstUnit::inp;
   using NstUnit::out;

   using NstUnit::inpInfo;
   using NstUnit::outInfo;

   using NstUnit::numInputs;
   using NstUnit::numOutputs;

   using NstUnit::getUnit;

private:
   NstMethodUnit (const char* pcLabel, unitptr uHost);
   virtual int exec ();
};

template<class T,  int (T::*pMethod) (unitptr)>
int ExecMethod (unitptr u) {
   T* pT = (T*) nst_var_of (u);
   return (pT->*pMethod)(u);
}
