#ifndef __NST_BASIC_H__
#   define __NST_BASIC_H__

typedef struct nst_wire_def_{
   int nr,    /* nr of references to this wire definition */
   iDimOut,   /* dimension of destination field at which this wire ends */
   *piSrcOf;  /* array of DimOut pin indexes.
                 SrcOf[i] = if >=0:
                              position of src pin in src field feeding to
                              destination pin at position i.
                           if -1:
                              dest pin at pos i has not yet any writer.
              */
   struct nst_wire_ **ppWireThatFeeds;
             /* if dest pin i has a writer pin, the connection need
                not be part of THIS wire. The wire to which a connection
                to dest pin i belongs is always given by WireThatFeeds[i].
             */
} nst_wire_def_t;

typedef struct nst_wire_{
   struct nst_wire_ *next,*prev;
   struct nst_inob_  *pSrc, *pDest; /* inob of src and dest unit */
   short int iSrcField, iDestField; /* pos nr of src and dest field */
   short int iSrcIsOut, iDestIsOut; /* type of src and dest field */
   short int iColor;  
   float  afWireGeom[6];  /* wire geometry info */ 
   float fX,fY,fLen;  /* geometry data */
   nst_wire_def_t   *psWireDef;     /* shared definition */
} nst_wire_t;

typedef struct nst_inob_{
	unitptr u;                /* backlink to instance */
	neo_icon_def_entry_t *pClass; /* class description */
	dialog_var_t *pCreateVar; /* variables in create dialog */
	dialog_var_t *pConfigVar; /* variables in config dialog */
	dialog_var_t *pUserVar;   /* variables in user create dialog */
	char *pcUserDialog;       /* if !NULL: create dialog is user-
								 defined and this is the format */
	int iUpdates;             /* update count */
	int   iNumWires;
	nst_wire_t  *pWire;       /* points to list of wire structs */
	int iAnnotationEntries;   /* nr of annotation entries in this unit */
	char *pcAnnotations;      /* their texts in a single piece */
	/*** fields holding tmp info: */
	char *pcInterfaceDesc;    /* set to private adr, don't free! */
	int iOpnds;
	/*** info only used by NEO: */
	int iXll, iYll;           /* screen info only kept for Neo */
	unsigned long iFlags;
	char *pcLabel, *pcName;
	char *pcClassOfRef;         /* if by-reference instance: this is the 
								   class name of the referenced unit, NULL else */
	int iNumVarnameDefs;
	char **ppcVarnameDef;		/* string array holding variable defs (if any) */
	unsigned int iId;			/* circuitwise unique integer > 0 */
} nst_inob_t;

#endif

