#ifndef __NST_SRVIO_H__
#define __NST_SRVIO_H__

#include "Neo_dialog.h"

typedef struct nst_wire_def_{
   int nr,    /* nr of references to this wire definition */
   iDimOut,   /* dimension of destination field at which this wire ends */
   *piSrcOf;  /* array of DimOut pin indexes.
                 SrcOf[i] = if >=0:
                              position of src pin in src field feeding to
                              destination pin at position i.
                           if -1:
                              dest pin at pos i has not yet any writer.
              */
   struct nst_wire_ **ppWireThatFeeds;
             /* if dest pin i has a writer pin, the connection need
                not be part of THIS wire. The wire to which a connection
                to dest pin i belongs is always given by WireThatFeeds[i].
             */
} nst_wire_def_t;

typedef struct nst_wire_{
   struct nst_wire_ *next,*prev;    /* wire chain */
   unitptr host;                    /* host unit of wire */
   struct nst_inob_  *pSrc, *pDest; /* inob of src and dest unit */
   short int iSrcField, iDestField; /* pos nr of src and dest field */
   short int iSrcIsOut, iDestIsOut; /* type of src and dest field */
   short int iColor;
   float fX,fY,fLen;                /* geometry data */
   nst_wire_def_t   *psWireDef;     /* shared definition of fibers */
} nst_wire_t;

typedef struct nst_inob_{
	unitptr u;                  /* backlink to instance */
	NstIconDefEntry_t *pClass;  /* class description */
	dialog_var_t *pCreateVar;   /* variables in create dialog */
	dialog_var_t *pConfigVar;   /* variables in config dialog */
	dialog_var_t *pUserVar;     /* variables in user create dialog */
	char *pcUserDialog;         /* if !NULL: create dialog is user-
								   defined and this is the format */
	int iUpdates;               /* update count */
	int   iNumWires;
	nst_wire_t  *pWire;         /* points to list of wire structs */
	int iAnnotationEntries;     /* nr of annotation entries in this unit */
	char *pcAnnotations;        /* their texts in a single piece */
	/*** fields holding tmp info: */
	char *pcInterfaceDesc;      /* set to private adr, don't free! */
	int iOpnds, iVirtualOpnds;
	/*** info only used by NEO: */
	int iXll, iYll;             /* screen info only kept for Neo */
	unsigned long iFlags;
	char *pcLabel, *pcName;
	char *pcClassOfRef;         /* if by-reference instance: this is the 
								   class name of the referenced unit, NULL else */
	int iNumVarnameDefs;
	char **ppcVarnameDef;		/* string array holding variable defs (if any) */
	unsigned int iId;			/* circuitwise unique integer > 0 */
        int iDisabled;

} nst_inob_t;

extern int skip_white_space(FILE*);
extern int neo_is_container(NstIconDef_t*);
extern int nst_is_dummy_inob(nst_inob_t *pInob);

extern void nst_fprintf_inob(FILE *fp, nst_inob_t *pDialog);
extern void nst_fprintf_dialog_var(FILE *fp, dialog_var_t *pDialog);

extern nst_inob_t   *nst_fscanf_inob(FILE *fp, nst_inob_t *pInob);
extern int           nst_fscanf_inob_error(void);
extern nst_inob_t   *nst_fscanf_global_var(FILE*,nst_inob_t*);
extern nst_inob_t   *nst_fscanf_global_var0(FILE*,nst_inob_t*);
extern dialog_var_t *nst_fscanf_user_var(FILE*,dialog_var_t*,char*);
extern dialog_var_t *nst_fscanf_dialog_var(FILE*,dialog_var_t*,dialog_var_t*);
extern dialog_var_t *nst_dialog_var_from_fmt(char*);

extern void nst_free_dialog_var_t(dialog_var_t*);
extern void nst_free_wire_info(nst_wire_t *pWire);
extern void nst_free_inob_t(nst_inob_t *pDialog);

extern int nst_exact_version(void);

#endif

