#ifndef __NEO_H
#define __NEO_H

/*********************************************************************

   NEO_HEADER_VERSION: modify value after major changes in header:   
                      
   Increment major (first) version nr, when change is certain to 
   lead to incompatibility with folders compiled before the change.
   Increment minor (second) version nr for less dramatic changes
   that, however, might be worth a warning.
   This allows Neo to safeguard against version mismatches with 
   dynamically loaded units.

 **********************************************************************/

#ifndef NEO_HEADER_VERSION
#   define NEO_HEADER_VERSION "7.9 17/07/02 HR"   
#endif

/************** public functions needed in wrappers: *****************/

#include "nst.h"

extern char *neo_set_iconlabel(char*);
extern int   neo_set_iconlabel1(char*);
extern void  neo_set_icon_commands(const char*);
extern FILE* NeoOpenFile(char *pcFileId, char **ppcFileUsed, char* pcMode);
extern char* (*neo_window_placement)(char*,char*,int,int,int,int);
extern char* (*SRCLINE)(char*,...);
extern char* (*pcTVAL)(unitptr,int );
extern int   NeoInteractiveMode (void);

#define THREAD_BOX_NAME "thread_box"

/************** Macros to simplify writing of wrappers: **************/

extern int HASVAR(int); /* indicator fnct for $/@ vars in text value */
extern int   IVAL(int); /* get i-th numeric value of dialog as int   */
extern float FVAL(int); /* get i-th numeric value of dialog as float */
extern char *TVAL(int); /* get ptr to i-th text value of dialog      

NOTE: written numeric values count as numeric and text simultaneously! */

/************* to define contents of a folder use these: ************/

#define BEGINFOLDER(NAME) static char gacThisFolder[]=#NAME; \
   char *Neo_V_of_##NAME[2] = {NEO_HEADER_VERSION,NST_HEADER_VERSION};\
   extern NstIconDefEntry_t gasIconsOfFolder_##NAME[];\
   NstFolder_t gsFolder_##NAME = {NULL, #NAME, gasIconsOfFolder_##NAME};\
   NstIconDefEntry_t gasIconsOfFolder_##NAME[]={
#define ICONENTRY(NAME) {gacThisFolder,#NAME,&ICONDEF_##NAME,NULL,NULL}
#define ENDFOLDER ,{"","",NULL,NULL,NULL}}

#define INIT(NAME) void _NeoInit_##NAME(void) 
#define FINI(NAME) void _NeoFini_##NAME(void) 

/************* to define each icon's dialog  use these: ************/

#define ICONFORM1(NAME)\
 extern void    ICONFUN(NAME,0) (int,int,NstIconDef_t*,unitptr);\
 static unitptr ICONFUN(NAME,1) (int,int,NstIconDef_t*,unitptr);\
 static int     ICONFUN(NAME,2) (int,int,NstIconDef_t*,unitptr);\
 NstIconDef_t   ICONDEF_##NAME = {NULL,ICONFUN(NAME,1),ICONFUN(NAME,2),

#define ICONFORM2(NAME)     STD_BUTTONS,
#define NO_ICONFORM2        "\0" 

#define ICONLABEL(LABEL)    STD_BUTTONS,LABEL,"" 
#define ICONMANPAGE(MANPGE) MANPGE              /* optional */ 

#define ICONINIT(NAME)      ,ICONFUN(NAME,0)};\
 void ICONFUN(NAME,0)(int NUMVAL,int NUMTXT,NstIconDef_t *_def,unitptr u){

#define ICONMAKE(NAME) }; \
 static unitptr ICONFUN(NAME,1) (int NUMVAL,int NUMTXT,NstIconDef_t *_def,unitptr HOST)

#define ICONPARS(NAME) \
 static int ICONFUN(NAME,2) (int NUMVAL,int NUMTXT,NstIconDef_t *_def,unitptr u)

/****** to define info texts for fields of a unit/method subunit: *******/

extern void SET_INFO_FOR_METHOD(char*);
extern void INFO_FOR_INP(int,char*,...);
extern void INFO_FOR_OUT(int,char*,...);
#define     SET_INFO_FOR_CONTAINER       SET_INFO_FOR_METHOD(NULL) 
 
/****** to define help texts for entire method subunits: ***************/

#define ICONFORM_FOR_METHOD(NAME) neo_iconform_for_method (#NAME, 
#define END , NULL, NULL);
#define ICONFORM2_FOR_METHOD(FUNC) , ICONFUN(FUNC,3), 
#define END2 );
#define ICONPARS_FOR_METHOD(FUNC) \
  static int ICONFUN(FUNC,3) (int NUMVAL,int NUMTXT,NstIconDef_t *_def,unitptr u)

/**** to define associated popup commands within ICONMAKE routine: ******/

#define ICONCOMMANDS(CMDSTR) neo_set_icon_commands(CMDSTR)

/**********************  internal stuff:  ****************************/

#define ICONFUN(NAME,I) _fun##I##_##NAME
#define STD_BUTTONS  " %[%0:1:0:10d %l`Cancel%d%l`Ok%]%H "
#define STD_BUTTONS0 " %[%0:0:0:10d %l`Cancel%d%l`Ok%]%H "
#define NEO_VERSION_SYMBOL "Neo_V_of_"

/************** Data types required in wrappers: *********************/

typedef struct NstIconDef_{
   void     *pPrivate;
   unitptr  (*pCreateFunction)(int,int,struct NstIconDef_*,unitptr);
   int      (*pConfigFunction)(int,int,struct NstIconDef_*,unitptr); 
   char     *pcCreateDialog, *pcConfigDialog, *pcDefaultLabel, *pcManpage; 
   void     (*pInitFunction)  (int,int,struct NstIconDef_*,unitptr);
} NstIconDef_t;

typedef struct NstIconDefEntry_{
   char *pcFolderName;   /* the folders print name */
   char *pcClassName;    /* the icons class name   */
   NstIconDef_t *pDef;   /* the icons definition   */
   struct NstFolder_t *pSubFolder;
   struct NstIconDefEntry_ *next; 
} NstIconDefEntry_t;

typedef struct NstFolder_t{
   void *pPrivate;
   char *pcFolderName;   /* the folders print name */
   NstIconDefEntry_t *psEntry; /* a pointer to the first el. in entry list */
} NstFolder_t;

extern void neo_iconform_for_method (char*, char*, int (*)(int NUMVAL,int NUMTXT,NstIconDef_t *_def,unitptr), char*);

#endif
