/******************************************************************************
 *
 * neo1.h  -  headers for neo1.c
 *
 * Copyright Helge Ritter, 1994
 *
 * $Source$
 * $Log$
 * Revision 1.23  2005/10/14 12:02:58  rhaschke
 * replaced calls to MakeCircuitWdwTitle by NeoDefaultCircuitWdwTitle
 *
 * Revision 1.22  2005/09/02 13:44:10  rhaschke
 * merge with Helge's NST7 version: simplified event loop, smaller neo-files
 * major cleanup of compiler switches, declarations, etc.
 *
 * Revision 1.16  2000/02/07 17:20:36  helge
 * - some adjustments to support new exec_container_opnds() call of prog_unit
 *   with interactively adjustable scope rectangle around container
 * - bug correction (inconsistent scope updates when several virtual_for_op's
 *   were in the same container)
 * - ManPage window no longer blocking
 *
 * Revision 1.14  1999/12/14 16:01:05  helge
 * Minor changes & bug corrections:
 * - new button in Prefs menue to set pin info display mode
 * - repaired window redraw after resize (was lost in previous change)
 * - Load command: file browser now allows longer file names
 * - together with changes to nst_scrn & nst_X11: removed "busy wait loops"
 *   that did occur in some situations during input.
 * - now prog unit is informed during loading about compiled-in Neo folders
 *   (allows #import from these when *FUNCTIONS_foldername[]={...} is def'd)
 *
 * Revision 1.12  1999/05/31 18:07:32  helge
 * V7.0: many improvements, among them better event handling, smaller neo.h,
 *       better Search command, Folder Groups, better display of pin info.
 *
 * Revision 1.8  1997/10/13 16:42:23  helge
 * - extensive clean-up in src-generator routines (mainly neo6.c)
 * - some bugs in neo remake_unit removed (neo11.c)
 * - better src generation, better diagnostics (neo1,2,main)
 * - some wrappers corrected   (mainly SRCLINEs)
 * - unit funct_gen now obsolete and hidden (but kept); use
 *   function_gen for new work (differs only in wrapper)
 *
 * Revision 1.5  1997/08/23 21:53:09  helge
 * Some changes/extensions:
 * 1. there is no longer NeoExmplPath and NeoManPath to be set in .neo
 *    or on the command line. Instead, if a folder is specified to be
 *    loaded from path/o.objectdir/folder.so, examples and manpages for
 *    this folder are now sought in path/examples, path/cat3 and path/html3.
 *    If no path is specified, or the folder is compiled, NeoHome is taken for path.
 * 2. When a circuit contains the new reserved token $LOAD_DIR in its
 *    text entry fields, this will be taken as the name of the directory
 *    from which the current circuit was loaded ("." if used in a circuit
 *    that was not loaded). This allows to reference files needed by a circuit
 *    in a way that is independent of the circuit's actual directory.
 *    For details, see log file.
 * 3. minor changes: redrawing of widget scopes, bugfix for src generation,
 *    Neo-Prefix for some routines.
 *
 * Revision 1.1  1995/03/06  15:17:19  helge
 * source files for neo kernel.
 *
 *****************************************************************************/
#ifndef __NEO1_H
#define __NEO1_H

#include "neo_types.h"
#include "neo_event.h"

/*************************** exports from neo1-1.c **************************************/

#define ICONBOXMARGIN  15     /* margin to be left when displaying icons */
#define LOWERHEIGHT 8         /* height of a lowercase char in the iconfont */
#define ICONTITLELEN 6 
#define WIRESTEPFUDGE 3          /* smallest toleratet step in wireshape */
#define WIREMEANDERTOLX 35       /* half width of narrowest tolerated meander */
#define WIREMEANDERTOLY  (giNstScreenDx/50)  
                                 /* half width of narrowest tolerated meander */
#define IFUDGE 15                 /* space between autoplaced windows */
#define SHIFT_OF_COPY  25         /* shift when a copy icon is made */
#define DYNARRAY_MINCOLOR  10000  /* larger than any other NST color index! */

/* global scaling factors for contents of ModuleWdw, CircuitWdw and
   WireEditWdw [default values of 1 give icon size as specified above
   in pixels] */
extern float gfNeoXScale;
extern float gfNeoYScale;

#define MAXLEVELS     100 /* max nesting levels of circuits */

#define NEO_XSCALE(A)  ((A)*gfNeoXScale)
#define NEO_YSCALE(A)  ((A)*gfNeoYScale)

extern unitptr guCurrentDefWdw;  /* the defwdw of the current box */

extern int giDoHide;          /* global hide mode */
extern int giPrefConfirm;     /* confirm major deletes ? */
extern int giPrefTrace;
extern int giPrefTraceLevel;

extern void ErrorHandler(int iWarnings, int iErrors);
extern void ErrorPrinter(char *pcErrMsg);

extern void NeoResizeNotify(int iWdw);
extern void NeoDrawArrow(int iOri, float xll, float yll, float xur, float yur, int iColor, int iDarkRim, int iBrightRim);
extern void NeoDrawBox(float xll, float yll, float xur, float yur, int cul, int clr);
extern void NeoDrawDashedFrame(float xll, float yll, float xur, float yur, int iCol);
extern void NeoDrawDashedFrame2i(int iXll,int iYll,int iXur, int iYur, int iCol);
extern void NeoDrawFields(unitptr uIcon, float fXoff, float fYoff, float fXscl, float fYscl);
extern void NeoDrawFrame(float xll, float yll, float xur, float yur, int iWidth, int iCol);
extern void NeoDrawLine(float iX1, float iY1, float iX2, float iY2, int iCol);
extern void NeoDrawNonintersectingFrame(float xll, float yll, float xur, float yur, int iCol, int iDashed, unitptr uHost);
extern void NeoDrawTile(float xll, float yll, float xur, float yur, int iCol, int iBrightRim, int iDarkRim);
extern void NeoDrawWireShape(float fX0, float fY0, float fX2, float fY2, NstWireShape_t *psWire, int iCol);
extern void NeoRectf(float xll, float yll, float xur, float yur);
extern void NeoRectf2(float xll, float yll, float xur, float yur, int iCol1, int iCol2);
extern void NeoRectf3(float xll, float yll, float xur, float yur, int iCol1, int iCol2, int iWidth);

extern int NeoExposeEventHandler(int iWdw);
extern void NeoGetIconPos(float *pfXll, float *pfYll, unitptr uIcon);
extern void NeoGridf(float xll, float yll, float xur, float yur);
extern int NeoIconIsByReference(unitptr uIcon);
extern int NeoKeyIsDown(char cKey);
extern void NeoMapWindowsOf(unitptr uIcon);
extern void NeoUnmapWindowsOf(unitptr uIcon);
extern void UpdateWdw(NstIconWdw_t *psWdw, int iNewMode);
extern int NeoPinColor(pin* pPin);
extern int getFieldColor(field *pF);
extern int NeoRedrawIconWdw(NstIconWdw_t *psWdw);
extern void NeoSetBusyCursor(void);

extern void NeoSetIcon(unitptr uIcon);
extern unitptr NeoGetIcon();
extern char* NeoSetIconLabel(char *pcNewLabel, unitptr uIcon);
extern void NeoSetNormalCursor(void);
extern void NeoTile3(float xll, float yll, float xur, float yur, int iCol1, int iCol2, int iWidth);
extern int getPlainFieldDim(field *pF);

/*********************** exports from neo1-2.c *******************************/

struct popup_data_t;
extern int getMouseState (int *piMx, int *piMy, int *piButton);

extern void NeoSetPopupSeparator (void);
extern void NeoShowPopup (int iMx, int iMy, struct popup_data_t *pData);
extern void NeoButtonReleaseForPopup (struct popup_data_t *pData);
extern int  NeoPointerMotionForPopup (neo_event_t *pEv, struct popup_data_t *pData);
extern char* NeoPopup (char *pcMenu, int iMx, int iMy, int iButton, int iWidth, int iHeight, unitptr uOrig);
extern int  NeoInsidePopup (int iMx, int iMy, struct popup_data_t *pData);
extern void NeoFreePopup (struct popup_data_t *pData);
extern void* NeoPopupMenuForUnit (char *pcMenu, int iMx, int iMy, 
                                  int iButton, int iWidth, int iHeight, unitptr uOrig);
extern char* NeoGetPopupResult (struct popup_data_t *pData);
extern struct popup_data_t* NeoDisplayFieldInfo (char *pcText, int iMx, int iMy, 
                                                 int iButton, field *pF1, unitptr u);

/*********************** exports from neo1-3.c *******************************/

extern FILE* fpPostScript; /* a global file pointer for postscript output */
extern int giFindPSDimensions ;

/*********************** exports from neo1-4.c *******************************/

extern int giNeoLeftMargin;
extern void NeoInitFunctions (void);
extern char* NeoPathFromFile (char *pcFile);
extern int NeoInitializeWindowPlacement (void);
extern void NeoReinitializeWindowPlacement (void);
extern void neo_finish_window_placement (unitptr uNew);
extern void NeoRemoveMenuUnit (unitptr uMenue);
extern void NeoEnableCompactMenu();
extern void NeoDisableCompactMenu();
extern int  NeoCompactMenu (void);
extern unitptr NeoCreateMenuUnit (int iNumCmd, const char **ppcCmd);
extern char* NeoFileExists (char *pcFileId, char *apcSuffixList[]);
extern int NeoCloseFile (FILE *fp, char *pcMode);
extern char* NeoHasSuffixFromList (char *pcFile, char **ppcSuffixList);

/*********************** exports from neo1-5.c *******************************/

extern void NeoInitErrorReporting (void);
extern void NeoEnableErrorReport (void);
extern void NeoDisableErrorReport (void);
extern int NeoErrorPrinterActive (void);
extern void ErrorPrinter (char *pcErrMsg);
extern void ErrorHandler (int iWarnings, int iErrors);
extern char* NeoCollectErrorMsg (char *pcMsg, int iMode);

/*********************** exports from neo1-6.c *******************************/

extern void NeoInitFontHalfWid (void);
extern char* mkBaseName (char *pcBaseName, char *pcName);
extern void mkFileNames (char *pcFile, char *pcCircuitFile, char *pcDataFile, char *pcInfoFile);
extern void mkLengthLimitedDescription(char *pcBuf, char *pcFmt, int iMaxLen);
extern int CountLinesInFmt (char *pcFmt);
extern int findWdwHeight (char *pcFmt);
extern int giDisplayIconTypes ;
extern void NeoToggleCircuitDisplay (void);
extern void NeoDrawTitle (float xll, float yll, char *pcTitle, int iCol2, int iNum, unitptr uIcon);
extern void NeoDefaultCircuitWdwTitle (void);
extern float fPinVal(field *pF, int i, char t, char tEl);

/*********************** exports from neo1-7.c *******************************/

extern void AlertBox(char *pcText);
extern unitptr ChooseBox(char *pcText);
extern unitptr InfoBox(char *pcText);
extern int NeoDialogToCreateIcon(unitptr uWdw);
extern int NeoDialogToCreateIcon1(unitptr uWdw, int iAutoClose);
extern int NeoQueryFile(unitptr uWdw);
extern int NeoQueryWdwValuesDiffer(unitptr uWdw1, unitptr uWdw2, int iOmit);
extern int QueryLoop(unitptr uWdw);
extern unitptr makeQueryWdw(char *pcTitle, char *pcFmt, int iPos);

#endif /* __NEO1_H */
