/******************************************************************************
 *
 * neo3.h  -  headers for neo3.c
 *
 * Copyright Helge Ritter, 1994
 *
 * $Source$
 * $Log$
 * Revision 1.7  1999/02/14 17:49:46  helge
 *  - neo_main.c icons can now become activated (executed) when the
 *    cursor moves into a window that has been associated with the
 *    icon before (specification occurs in Define dialog).
 *  - neo.h neo1.c neo2.c neo3.c neo4.c neo5.c neo8.c neo9.c neo10.c
 *    neo_main.c : changes to allow loading of icon units from loadable
 *    folders "by reference": such icons cannot be changed internally
 *    and they will always stay in sync with their definition in
 *    the folder.
 *  - neo_main: changed color initialization to newer rgb:rr/gg/bb
 *              format to avoid problems with the initial '#'-symbol (which
 *              unfortunately is already reserved as a comment symbol).
 *  - optimization of instantiation of loadable icons from folders:
 *    so far, a loadable icon was made with its default parameters,
 *    and then remade when changes were requested. Now the query for
 *    changes is made earlier (in NeoReadHeaderIcon()) so that changes
 *    can already become effective during the first instantiating
 *    process and no further remake cycle is necessary.
 * - minor cleanup: EditWire() is now NeoEditWire() (affects
 *   neo?.c, neo3.h, neo_stdr.c)
 *
 * Revision 1.3  1997/04/02 12:46:57  helge
 * several minor corrections, among them: better handling of operator scopes,
 * ~-expansion in .neofolders-file, unloading of dynamic folders now works,
 * save gz file now safeguarded against specification of non-existent dir.
 *
 *****************************************************************************/
#ifndef __NEO3_H
#define __NEO3_H

extern int giPreserveOrigWires;
extern unitptr guSelectedWire;

extern unitptr wire_unit(unitptr,unitptr,field*,field*,int,int,unitptr);
extern int draw_wire_unit(unitptr,int,int);
extern int exec_wire_unit(unitptr);
extern int adapt_wire_unit(unitptr);
extern int ctrl_wire_unit(int,unitptr);
extern int load_wire_unit(FILE*,unitptr);
extern int save_wire_unit(FILE*,unitptr);
extern int remove_wire_unit(unitptr);

extern void getFieldsFromWireVarptr(field**,field**,var_wire_t*);

extern int  isWireUnit(unitptr u);
extern int  NeoShowWire(void);
extern void NeoRedrawWires(void);
extern void UndrawVoidWires(unitptr,unitptr);
extern void NeoUndrawMovedWires (void);
extern int  NeoStraightenMovedWires (void);
extern void move_wire (unitptr u, int iDx, int iDy);

extern int isLegalConnection(unitptr u1, unitptr u2, int iOut1, int iOut2);
extern int isConnected(unitptr uIcon, unitptr uWire);
extern unitptr WireBetween(field *pF1, field *pF2, unitptr uHostIcon);

extern void straightenWireShape(float fX0, float fY0, float fX2, float fY2,
                                NstWireShape_t *psWire, int iDraw);
extern int adjustWireShape(float fX0, float fY0, /* wire start and end */
                           float fX2, float fY2, 
                           NstWireShape_t *psWire,
                           int iMx, int iMy, /* cursor pos */
                           int iMode, int iTol);

extern unitptr NeoWireCanBeCopied(unitptr uSrcIconNew, unitptr uDestIconNew,
                                  field *pFSrcNew, field *pFDestNew,
                                  unitptr uWire, unitptr uHost);
extern unitptr InstallCopyOfWire(unitptr,unitptr,unitptr,unitptr);
extern void RmSelectedWireIfVoid(void);
extern int  ModifyFieldForWire(unitptr,int,int,unitptr);
extern void NeoModifyOrAddField (int iPos, char *pcFmt, int isOut, 
                                 unitptr uIcon);
extern int DescribeWire(unitptr);

extern int adapt_wire_box (unitptr u);

extern void NeoImproveWiresOf (unitptr uIcon);

#endif
