/*****************************************************************************
 *
 * nst5.h  -  NST kernel layer 5: Execution
 *
 * Copyright Helge Ritter, 1992
 *
 * $Source$
 *
 * $Log$
 * Revision 1.17  2006/12/22 11:31:44  rhaschke
 * added const char* here and there
 *
 * Revision 1.16  2005/11/23 14:47:19  rhaschke
 * simplified nst_get/release_graphics using recursive mutex
 *
 * Revision 1.15  2005/09/02 13:57:34  rhaschke
 * cleanup in declarations
 * nst_[scrn,X11,prog]: adjustments and additions for NST7's neosrc
 *
 * Revision 1.14  2003/02/07 10:25:13  jontrup
 * (new feature): Added function nst_abort_execution(). By calling this function
 * an application running a Neo circuit via the Nstloader might cancel the current
 * Neo execution thread.
 *
 * Revision 1.13  2002/08/12 07:01:51  rhaschke
 * added declarations of exported functions
 *
 * Revision 1.12  2002/08/08 09:35:29  jontrup
 * Added some #ifdef NST_THREADS compiler switches to allow compilation
 * of Neo/NST with or without thread support.
 *
 * Revision 1.11  2002/07/17 10:42:08  helge
 * added declarations for new thread handling functions
 *
 * Revision 1.10  2000/10/16 15:33:02  helge
 * (minor change) moved declaration of gfNstNaN from nst5 to nst0
 *
 * Revision 1.9  2000/06/24 19:21:44  helge
 * modified nst_throw() so that it can be used uniformly in NST and prog unit
 * #import lib code.
 *
 * Revision 1.8  2000/04/07 15:01:14  helge
 * added routines and definitions for NST exception handling:
 * now nst_throw() can "throw" a NST exception that can be caught by
 * other NST units when these are implemented as exception handlers
 *
 * Revision 1.7  2000/01/10 17:06:06  helge
 * added code to allow connecting scalar src vector to dyn float dest pin
 * (is enabled when S2DYN is #defined)
 *
 * Revision 1.6  1997/12/21 15:50:43  helge
 * minor adjustments
 *
 * Revision 1.5  1997/04/02 14:07:23  helge
 * most man pages now in source code; some corrections;
 * for list, check out new log file in main dir
 *
 * Revision 1.4  1997/03/20  09:31:54  jan
 * Missing 'extern' declarations added.
 *
 * Revision 1.3  1997/03/18 13:51:07  jan
 * Added missing 'extern' declarations.
 *
 * Revision 1.2  1997/03/13 10:48:40  jan
 * Dependency cleanup.
 *
 *
 *****************************************************************************/

#ifndef __NST5_H
#define __NST5_H
#include <stdio.h>
#include "nst_types.h"

/* these two in nst_method.c, but needed in many places: */
extern void nst_unname_unit(const char*,unitptr);
extern void nst_name_unit(const char*,unitptr);

extern  unitptr ROOT;
extern  unitptr SECRETROOT;
extern int error_checking(unitptr,int);

/* Predefined field identifiers:  */

extern  int in_0[2],in_1[2],in_2[2],in_3[2],in_4[2],in_5[2],in_6[2],in_7[2], in_8[2],in_9[2],X_in[2],Y_in[2],Z_in[2],X_out[2],Y_out[2],Z_out[2];
extern  int out_0[2],out_1[2],out_2[2],out_3[2],out_4[2],out_5[2],out_6[2],out_7[2],out_8[2],out_9[2];

extern  int in_[MAXFIELDS][2], out_[MAXFIELDS][2];
	/* provide indexable field identifiers in_[0], in_[1] etc. */

/* two special field identifiers: */

extern  int EPS_in[2], CTL_in[2];

#define NOTHING_TO_SAVE(U) ((U)->nvar == 0 && (U)->noutputs == 0)

extern  int exec_unit(unitptr);
extern  int remove_unit(unitptr);
extern  int adapt_unit(unitptr);
extern  int load_unit(FILE*,unitptr);
extern  int save_unit(FILE*,unitptr);
extern  int ctrl_unit(int,unitptr);
extern  int init_unit(unitptr);
extern  int clr_unit(unitptr);
extern  void init_nst(void);   /* obsolete name */
extern  void nst_init(void); 
extern  void nst_do_all_input_type_conversions(unitptr);
extern  void nst_do_all_output_type_conversions(unitptr);
extern  int (*nst_exit_routine)(char*);
extern  void set_nst_exit_routine(int (*user_routine)(char*));
extern  int  nst_plugin_mode(void);
extern  void nst_set_plugin_mode(int); 
extern  void nst_abort_execution();
extern  int  nst_default_exit(char*);
extern  int  giNstExitFlag;

/* routines for NST thread handling: */

extern int nst_is_thread_box(unitptr);
extern int nst_is_active_thread_box(unitptr);
extern int nst_is_main_thread(void);
extern int nst_infer_thread_safety_for(unitptr);

#ifdef NST_THREADS
extern void nst_get_graphics(void);
extern void nst_release_graphics(void);
#else
#define nst_get_graphics(x)
#define nst_release_graphics(x)
#endif
/* routines for NST runtime exception handling: */

extern int   nst_throw(const char*,const char*,...);
extern void (*nst_throw_hook)(const char*, const char*);
extern void  nst_clear_exception(void);
extern int   nst_is_exception_handler(unitptr);
extern void  nst_mark_as_exception_handler(unitptr);
extern const char *nst_errmatch(const char*);
extern int   nst_errmatch2(const char*,const char*);
extern char *nst_errtype(void);
extern char *nst_errtext(void);
extern char *nst_errtype_r(void);
extern char *nst_errtext_r(void);
extern void  nst_errtrace(FILE *fp);
extern void  nst_set_errunit(unitptr);
extern unitptr nst_errunit(void);


#ifdef NST_THREADS
#define SERVERSTUFF

#ifdef SERVERSTUFF
extern void nst_enable_server_mode(void);
extern void nst_disable_server_mode(void);
extern int  nst_server_mode(void);
extern void nst_reserve_access(void);
extern void nst_release_access(void);
#endif
#endif /* NST_THREADS */

#endif /* __NST5_H */
