/*****************************************************************************
 *
 * nst7.h  -  NST kernel layer 7: File-I/O for local variables of units
 *
 * Copyright Helge Ritter, 1992
 *
 * $Source$
 *
 * $Log$
 * Revision 1.4  2002/03/04 15:38:41  rhaschke
 * new exported functions: load_unit, load_container, save_unit,
 * nst_actual_load_section, nst_read_until_endmark
 *
 * Revision 1.3  1997/04/02 14:07:35  helge
 * most man pages now in source code; some corrections;
 * for list, check out new log file in main dir
 *
 * Revision 1.2  1997/03/13  10:48:46  jan
 * Dependency cleanup.
 *
 *
 *****************************************************************************/

#ifndef __NST7_H
#define __NST7_H
#include <stdio.h>
#include "nst_types.h"

extern int iCopyComments(FILE *fpDest, char * tCom, int iComMaxLen, FILE *fp, char cComChar);	
/* Utility function to copy comment lines (beginning with a char .cComChar)
   from .fp to .fpDest (if fpDest!=NULL) 
   and parallel max .iComMaxLen char to .tCom (if tCom!=NULL).
   WARNING: .tCom is null terminated and tCom[iComMaxLen] might be set to \0;
   Leading white space is ignored.
   Returns # of lines skipped. <JW 02-94>
   
   EXAMPLES:
    iCopyComments(stdout, NULL, 0, fp, '#');  /- copy to stdout -/
    iCopyComments(NULL, acTmp, 42, fp, '#');  /- copy max 42 char to acTmp[] -/
    iCopyComments(NULL, NULL, 0, fp, '#');    /- same as skip_comments -/
*/


extern  int save_unit(FILE *fp, unittype *u);
extern  int ivwrite(FILE*, char*, unitptr);

extern  int nst_skip_data_section(FILE*,unitptr);
extern  int load_unit(FILE *fp, unittype *u);
extern  int load_container(FILE *fp, unittype *u);
extern  int ivread(FILE*, char*, unitptr);

extern  char* nst_actual_load_section ();
extern  char *nst_read_until_endmark (FILE *fp, char *pcSection);

/* initialize fvget and read header information about
unit u (i.e. its name, # of input and output fields and
their pin numbers and the current variable values (using
format fmt) of the output pins). 
Return value: 1 indicates normal operation, 
             -1 value indicates end of file encountered
             other neg value indicates file format error.  */

extern  int fvput(FILE*, char*, char*, unitptr);

/* Returns, if current entry of variable queue of unit
u is marked as "forbidden" (size=0). Otherwise,
writes character string "text" to file fp, followed
by all variables that belong to current entry in variable
queue. String fmt specifies the format that is used.
Allowed values are one of "%n.mf","%f", "%d", "%c", "%s".
  In both cases, after return the variable-queue-pointer
of unit u points to the next queue entry.
I.e., using a series of calls of fvput, all variables
of the variable queue of unit u can be written to file fp.
Return value: 1 indicates normal operation, 
neg value indicates error. */


extern  int fvget(FILE*, char*, unitptr u);

/* Returns, if current entry of variable queue of unit
u is marked as "forbidden" (size=0). Otherwise,
writes character string "text" to file fp, followed
by all variables that belong to current entry in variable
queue. String fmt specifies the format that is used.
Allowed values are one of "%n.mf","%f", "%d", "%c", "%s".
  In both cases, after return the variable-queue-pointer
of unit u points to the next queue entry.
I.e., using a series of calls of fvput, all variables
of the variable queue of unit u can be written to file fp.
Return value: 1 indicates normal operation, 
             -1 value indicates end of file encountered
             other neg value indicates file format error.  
*/

#endif
