/*************************************************************************
 
 NST, Version 0.0
 source file nst_aux4.h

 Contains headers for routines to read/write pixel data from/to files,
 using various formats. 

 *************************************************************************/

#ifndef __NST_AUX4_H
#define __NST_AUX4_H

#include <stdio.h>

/*--------------------------------------------------------------------------*/

extern int write_pgm_file(char*,char*,float*,int,int);
extern int write_ppm_file(char*,char*,float*,float*,float*,int,int);
extern int write_pgm_file8(char*,char*,unsigned char*,int,int);
extern int write_ppm_file8(char*,char*,unsigned char*,unsigned char*,unsigned char*,int,int);
extern int read_pgm_file(char*,char*,float*,int*,int*);
extern int read_ppm_file(char*,char*,float*,float*,float*,int*,int*);
extern int read_pgm_file8(char*,char*,unsigned char*,int*,int*);
extern int read_ppm_file8(char*,char*,unsigned char*,unsigned char*,unsigned char*,int*,int*);
extern int nst_read_pnm_file(FILE*,int,int,int,int,float*,float*,float*);
extern int nst_read_pnm_file8(FILE*,int,int,int,int,
                              unsigned char*,unsigned char*,unsigned char*);

extern int nst_fprint_byte_pnm(FILE*,char*,unsigned char*,unsigned char*, unsigned char*,int,int);
extern int nst_fprint_float_pnm(FILE*,char*,float*,float*,float*,int,int);
extern int nst_fscan_byte_pnm(FILE*,char*,unsigned char*,unsigned char*, unsigned char*,int*,int*);
extern int nst_fscan_float_pnm(FILE*,char*,float*,float*,float*,int*,int*);
extern int nst_fscan_byte_pgm(FILE*,char*,unsigned char*,int*,int*);
extern int nst_fscan_float_pgm(FILE*,char*,float*,int*,int*);
extern int scanf_pnm_header(FILE*,char*,char*,int*,int*);

extern void nst_ps_flush(FILE*);
extern void nst_ps_linewidth(FILE*,float);
extern void nst_ps_border(FILE*);
extern void nst_ps_color(FILE*,int);
extern void nst_ps_rectwrite(FILE*,int,int,int,int,int,
              int,int,int,int,int,void*);
extern void nst_ps_colormap_rectwrite(FILE*,int,int,int,int,int,
              int,int,int,int,int,void*);
extern void nst_ps_rgb_rectwrite(FILE*,int,int,int,int,int,
              int,int,int,int,int,void*,void*,void*);
extern void nst_ps_rgbtriples_rectwrite(FILE*,int,int,int,int,int,
              int,int,int,int,int,void*);
extern void nst_ps_dashed(FILE *,int);
extern void nst_ps_cmov2(FILE*,float,float);
extern void nst_ps_cmov2i(FILE*,int,int);
extern void nst_ps_cmov(FILE*,int,int);
extern void nst_ps_move2(FILE*,float,float);
extern void nst_ps_circ2(FILE*,float,float,float);
extern void nst_ps_circ2i(FILE*,int,int,int);
extern void nst_ps_rmv2(FILE*,float,float);
extern void nst_ps_draw2(FILE*,float,float);
extern void nst_ps_rdr2(FILE*,float,float);
extern void nst_ps_pnt2(FILE*,float,float);
extern void nst_ps_move2i(FILE*,int,int);
extern void nst_ps_rmv2i(FILE*,int,int);
extern void nst_ps_draw2i(FILE*,int,int);
extern void nst_ps_rdr2i(FILE*,int,int);
extern void nst_ps_pnt2i(FILE*,int,int);
extern void nst_ps_rect(FILE*,float,float,float,float);
extern void nst_ps_rectf(FILE*,float,float,float,float);
extern void nst_ps_recti(FILE*,int,int,int,int);
extern void nst_ps_rectfi(FILE*,int,int,int,int);
extern void nst_ps_charstr(FILE*,char*);
extern void nst_ps_init(FILE*,float,float,float,float,float,float,char*);
extern void nst_ps_fillpath(FILE*);
extern void nst_ps_closepath(FILE*);

extern char *nst_escape_charset(char *pcStr, char *pcEscSet, char cEscChar);

/*--------------------------------------------------------------------------*/
int append_postscriptfile(char fileid[], float xllcm, float yllcm, float dxcm,
float dycm, float pix[], int dx, int dy);

int append_color_postscriptfile(char*, float, float, float, float, 
float*, float*, float*, int, int);

int append_eps_postscriptfile(char fileid[], float xllcm, float yllcm, 
     float dxcm, float dycm, float pix[], int dx, int dy);

int append_eps_color_postscriptfile(char*, float, float, float, float, 
float*, float*, float*, int, int);

/*--------------------------------------------------------------------------*/
int write_postscriptfile(char fileid[], float xllcm, float yllcm, float dxcm,
float dycm, float pix[], int dx, int dy);

int write_color_postscriptfile(char*, float, float, float, float, 
float*, float*, float*, int, int);

/* generates a postscript file from a pixel array.

   fileidi     : name of file to be generated
   xllcm,yllcm : location of lower left corner of image relative to
                 lower left corner of page in units of cm.
   dxcm, dycm  : width and height of image in units of cm.
   pix[]       : array holding pixel intensity values of image.
   dx,dy       : width and height in pixels of image region.

*/

/*--------------------------------------------------------------------------*/

void open_postscript(FILE *fp);

/* writes header for postscript data to file fp. fp must
   point to file that has been opened for writing before.
   This routine must be used once before any output to
   file fp is made with routine write_postscript(..).
*/

/*--------------------------------------------------------------------------*/

void write_postscript(FILE *fp, float xllcm, float yllcm,
float dxcm, float dycm, float pix[], int dx, int dy);

void write_color_postscript(FILE*, float, float, float, float, 
float*, float*, float*, int, int);

/* appends postscript description of pixel array pix to file fp.
 
   fp          : filepointer of output file
   xllcm,yllcm : location of lower left corner of image relative to
                 lower left corner of page in units of cm.
   dxcm, dycm  : width and height of image in units of cm.
   pix[]       : array holding pixel intensity values of image.
   dx,dy       : width and height in pixels of image region.

*/

/*--------------------------------------------------------------------------*/

void close_postscript(FILE *fp);

/* writes trailer to postscript file fp and closes it. */

/*--------------------------------------------------------------------------*/

void open_eps_postscript(FILE *fp, float xllcm, float yllcm,
float dxcm, float dycm);

/* writes header for postscript data to file fp. fp must
   point to file that has been opened for writing before.
   This routine must be used once before any output to
   file fp is made with routine write_postscript(..). The
   parameters define the bounding box.
*/

/*--------------------------------------------------------------------------*/

void write_eps_postscript(FILE *fp, float xllcm, float yllcm,
float dxcm, float dycm, float pix[], int dx, int dy);

void write_eps_color_postscript(FILE*, float, float, float, float, 
float*, float*, float*, int, int);


/* appends postscript description of pixel array pix to file fp.
 
   fp          : filepointer of output file
   xllcm,yllcm : location of lower left corner of image relative to
                 lower left corner of page in units of cm.
   dxcm, dycm  : width and height of image in units of cm.
   pix[]       : array holding pixel intensity values of image.
   dx,dy       : width and height in pixels of image region.

*/

/*--------------------------------------------------------------------------*/
int write_eps_postscriptfile(char *fileid, float xllcm, float yllcm,
float dxcm, float dycm, float pix[], int dx, int dy);

int write_eps_color_postscriptfile(char *, float, float,
float, float, float*, float*, float*,  int, int);

void close_eps_postscript(FILE *fp);

/* writes trailer to postscript file fp and closes it. */

#endif
