/*************************************************************************

   Version 2.0 of NST source file  nst_dbug.h
   Copyright by Helge Ritter 1992

   $Source$
   $Log$
   Revision 1.9  2005-11-23 13:47:47  rhaschke
   removed NstErrorPrinterActive

   Revision 1.8  2005/09/02 13:57:34  rhaschke
   cleanup in declarations
   nst_[scrn,X11,prog]: adjustments and additions for NST7's neosrc

   Revision 1.7  2004/10/13 12:54:35  rhaschke
   two bug fixes:
   - buffer overrun in error formatting routine caused crashes
   - multiply entering the error handler caused a deadlock
     (even with a single thread)
     the corresponding mutex is recursive now and an additional variable
     checks for multiple usage of the NST_ERROR_PRINTER

   Revision 1.6  1997/12/21 15:50:44  helge
   minor adjustments

   Revision 1.5  1997/12/21 00:03:35  helge
   new routines nst_eprintf(), nst_veprintf() for diagnostics [mainly
   to be used in conjunction with mku+ and mku++, cf. mkudoc.tex]

**************************************************************************/

#ifndef __NST_DBUG_H
#define __NST_DBUG_H

#include <stdarg.h>
#include "nst_types.h"

extern  int ERRORS_FOUND;
extern  int ERROR_LINE;

extern  char *nst_error_msg[];

extern void nst_no_current_unit(void);
extern unitptr nst_current_unit(void);
extern int (*nst_set_user_hook(int (*)(int,unitptr)))(int,unitptr);

extern void traceline(char *s, int l);

extern void describe_pin(pin* p);

extern void describe_queue(unittype* u);

extern int describe_unit(unittype *u, int level);

extern void diagnose_connectivity(unittype *u);

extern int set_nst_errorhandler(void (*USER_ERROR_PRINTER)(char*),
                         void (*USER_ERROR_HANDLER)(int,int)); 

extern int error_msg(int, const char*, int, const char*);

/* this macro sets global var NST_TRACE to the trace level of unit U: */
#define NST_SET_TRACE_LEVEL_OF(U) \
   if ((NST_TRACE=((U)->property & P_trace_level)) > 13) NST_TRACE -= 16
/* NOTE: trace levels <0 are special and switch of any warnings (<0) and
         any error messages (<-1). The -16 is needed since we extract the
         trace level info from a nonegative int bit field. 
*/
extern int nst_trace_level_of(unitptr u);

extern int nst_trace(int,unitptr);
extern void nst_message(int, const char*,...);
extern void nst_warning(const char*,...);
extern void nst_error(const char*,...);
extern int nst_diagnostics_type(void);

extern int nst_eprintf(FILE*, const char*, ...);
extern int nst_veprintf(FILE*, const char*, va_list);
extern int nst_wprintf(FILE*, const char*, ...);
extern int nst_vwprintf(FILE*, const char*, va_list);

extern int error_report(void);
extern int error_checking(unitptr, int);

#endif /* __NST_DBUG_H */
