/*************************************************************************

   Version 2.0 of NST source file nst_pixl.c
   NST-routines to handle pixel images and windows

   Copyright by Helge Ritter 1992

 * $Source$
 * $Log$
 * Revision 1.5  2002/03/15 15:34:23  helge
 * adapted window units to changes in NST graphics layer.
 * write_pix:  additional input types:  dynamic pins (window will guess
 * rectangle shape or allow choice of width at pin 1), and single
 * packed field for RGB triples (described in Neo wrapper)
 * wit
 *
 * Revision 1.4  2001/08/03 13:55:45  jontrup
 * Included a slightly modified version of T. Twellmanns grab_wdw
 * unit into the pixel folder. It allows to grab named NST windows
 * and to write their contents as ppm or gif files.
 *
 * Revision 1.3  1999/05/31 18:17:02  helge
 * *** empty log message ***
 *
 * Revision 1.2  1998/01/05 18:24:44  helge
 * minor change to wrapper
 *
 * Revision 1.1  1997/08/30 18:16:35  helge
 * Made some former compiled folders now independent, dynamic folders.
 *
 * Revision 1.4  1997/04/02  14:09:35  helge
 * most man pages now in source code; some corrections;
 * for list, check out new log file in main dir
 *
 * Revision 1.2  1997/03/13  10:55:32  jan
 * Dependency cleanup.
 *
 * Revision 1.18  1997/01/20 13:48:39  helge
 * various minor changes.
 *
 * Revision 1.17  1997/01/14  14:26:10  helge
 * numerous small changes to take care of (almost) all warnings under
 * -Wall -O2.
 *
 * Revision 1.16  1996/08/07  14:23:10  helge
 * New NST-Version 4.8 / Neo-Version 2.8  accumulating major changes
 *
 * Revision 1.15  1995/07/31  09:23:12  helge
 * some minor changes.
 *
 * Revision 1.13  1995/07/16 20:55:13  helge
 * enhanced rpc_unit to make remote units from neo's *.NST-files
 * extended nst_rpd.c, nst_rpd_add accordingly [changes disabled by
 * undefing NEO in nst_rpd_add.c]
 * provided 8-bit versions of some nst-units [nst_math8 etc.]
 *
 * Revision 1.12  1995/03/07  08:09:18  helge
 * adding minor changes.
 *
 * Revision 1.11  1995/03/06  12:42:49  helge
 * (intermediate check in, final version follows shortly)
 *
 * Revision 1.10  1994/07/29  16:12:07  helge
 * *** empty log message ***
 *
 * Revision 1.9  1994/06/05  15:44:41  littmann
 * Image filter routines have been moved to nst_filt.[ch].
 *
 *
**************************************************************************/

#ifndef __NST_PIXL_H
#define __NST_PIXL_H

#include "nst_aux5.h"

/* convention for options to encode color modes: */

#define NST_RGBMODE      00200   /* to test for RGB-mode */
#define NST_R_OPTION     04200   /* only Red, RGB-mode */
#define NST_G_OPTION     02200   /* only Green, RGB-mode */
#define NST_B_OPTION     01200   /* only Blue, RGB-mode */
#define NST_RGB_OPTION   00600   /* all colors RGB, RGB-mode */
#define NST_COLOR_OPTION 00400   /* all colors RGB, but color map mode */
#define NST_GRAY_OPTION  00000   /* only gray levels, color map mode */
#define NST_C_OPTION     00010   /* mono, but color map mode */

/* Remark: the lowest two bytes are free for other usage,
   the next two bytes are bit-wise used to indicate usage of:
   RedOnly  GreenOnly  BlueOnly  RGBTogether  RGBMode  unused
*/

typedef struct var_write_pix{
   nst_winfo_t *pWinfo;
   int isSubwindow;
   int isOpen;
   int *piPosInfo;
   int iWidth, iPixels;
} var_write_pix_t;

extern int nst_is_write_pix_unit(unitptr); 

extern unitptr write_pix(int dx, int dy, char *wdw, char *fmt, unitptr dest);
/* creates unit that displays pixel values at its input pins as a
bitmap in a window described by string wdw, which must have the following
format:

   wdw = "title xll yll scale-factor"

where "title" is the title used for the window and must be
uninterrupted by white space, (xll,yll) is the location of
the lower left corner of the window in pixel coordinates on
the screen, and scale-factor determines
the size of the window in pixels according to

   width = scale-factor * dx
   height = scale-factor *dy

If fmt = NULL, the created unit will have a single packed input field
of dx*dy pins the values of which (using the interval [0..255])
are displayed as gray levels.

If fmt = "RGB" or "rgb", the created unit will have three packed input
fields of dx*dy pins each and will display the input values as an RGB-
color-image.   

IF fmt = "R", "G" or "B", the created unit will have a single packed
input field of dx*dy pins and render the input values as single color
image of the specified color.

*/



extern unitptr read_pix(int dx, int dy, int wdw, char *fmt, unitptr dest);

/*  input fields:
    X_in[2]  :  offset of rectangular rectangular read-area
                from lower left corner of RGBwindow wdw.
                (default value is (0,0) )
    output fields:
    Y_out[dx*dy] : packed field holding pixel intensities
                of rectangle of dx*dy pixels.
*/

extern unitptr locate_peaks(int,int,int,int,int,char*,unitptr);

extern unitptr centroid(int dx, int dy, unitptr dest);

/* computes center of gravity (cx,cy)
   of gray level image.
   One packed input field X_in[dx*dy]
   output field        X_out[2]    cx, cy
*/


extern unitptr combine(int dx1, int dy1, int dx2, int dy2, int mode, unitptr dest);

/* combine two image regions */


extern unitptr remap(int dx1, int dy1, int dx2, int dy2, unitptr dest);

/* maps image rectangle 1 onto image
   rectangle 2 by averaging intensity values within
   pixel blocks in image 1, or mapping pixels of image 1
   to pixel blocks of image 2, depending on which
   image is smaller.

   X_in[dx1*dy1]  : packed input field for input image 1
   Y_out[dx2*dy2] : packed output field for output image 2
   dx1,dy1: width and height in pixels of input image
   dx2,dy2: width and height in pixels of output image

   The lower left corner of the input image yields the
   value assigned to the lower left pixel of the output image.
   i.e. both image regions are aligned at their lower left corners.
*/
extern unitptr reduce_nonintegral(int magx, int magy, unitptr dest);
/* reduces image rectangle 1 onto image
   rectangle 2 by averaging intensity values within
   pixel blocks in image 1.

   X_in[]  : packed input field for input image 1
   dx1,dy2: input image size
   Y_out[] : packed output field for output image 2
   dx2,dy2: output image size, determined from integral parts of
                                   dx1/magx, dy1/magy

*/

extern unitptr subrect(int dx1, int dy1, int dx2, int dy2, unitptr dest);

/* X_in[dx1*dy1]  : packed input field, representing dx1*dy1 pixel image
   Y_out[dx2*dy2]  : packed output field, representing dx2*dy2 subregion
                    of input image.
   Y_in[2] : lower left corner of subregion within input image.
*/
extern unitptr subrect_dyn(unitptr dest);

/* X_in[dx1*dy1]  : packed input field, representing dx1*dy1 pixel image
   Y_out[dx2*dy2]  : packed output field, representing dx2*dy2 subregion
                    of input image.
   Y_in[2] : lower left corner of subregion within input image.
   dx1,dy1  size of first image
   dx2,dy2  size of second image
*/

extern unitptr subrect_of_change(int, int, float, char*, unitptr);

extern unitptr spatial_operator(
   int iNumIm,   /* nr of input images */
   int iDx,     /* width of image rectangle */
   int iDy,     /* height of image rectangle */
   int iDx1,    /* width of subrectangle */
   int iDy1,    /* height of subrectangle */
   int iDim,    /* output dim of mapped unit */
   int iNumPos, /* nr of subrectangle positions */
   int *piPos,   /* array of .iNumPos positions */
   char *tOptStr,  /* option string */
   unitptr uHost    /* host unit */
);

extern unitptr centroid(int dx, int dy, unitptr dest);
extern int ctrl_write_RGB_pix(int,unitptr);
extern int exec_locate_peaks(unitptr);
extern unitptr color_code(int,unitptr);
extern unitptr image_store(int,int,int,char*,unitptr);

extern unitptr locate_peaks8(int,int,int,int,int,char*,unitptr);
extern unitptr combine8(int dx1, int dy1, int dx2, int dy2, int mode, unitptr dest);
extern unitptr remap8(int dx1, int dy1, int dx2, int dy2, unitptr dest);
extern unitptr reduce_nonintegral8(int magx, int magy, unitptr dest);
extern unitptr subrect8(int dx1, int dy1, int dx2, int dy2, unitptr dest);
extern unitptr subrect_dyn8(unitptr dest);
extern unitptr write_pix8(int,int,char*,char*,unitptr);
unitptr centroid8(int dx, int dy, unitptr dest);
extern int exec_write_pix8(unitptr);
extern unitptr arrange_hexagonal(int,int,int,char*,unitptr);

extern unitptr grab_wdw(char* strWdwName, unitptr);


#endif
