typedef struct function_{
    char *pcName;         /* text name of this function                   */
    char *pcTypeOfArg;    /*  string encoding argument types of this 
                              function: '.' = float, '*' = float*.        */
    int  iReturnType;     /*  0 = void, 1 = float, 2 = implicit int       */
    int  iStart, iEnd;    /* offset to first and behind last instruction  */
    int  iNumArgVar;      /* nr of argument variables                     */
    int  iNumLocalVar;    /* nr of local variables of this function       */
    int  iNumStaticVar;   /* nr of static variables of this function      */
    int  iSizeOfAutoVar;  /* nr of bytes required for auto variables      */
    float *pfAutoVar;     /* pointer to storage block holding a set of
                             initialized automatic variables. A copy of
                             these will be made into the context_t record's
                             pfAutoVar, whenever a call to the function 
                             is made. */
    int  iMaxFloatEl;     /* max nr of float el held in pfAutoVar         */
    int  iMaxEntries;     /* nr of entries in pfAutoVar                   */
    int  *piOffset;       /* offsets of local var w.r.t pfAutoVar         */
    char **ppcLocalVar;   /* ppcLocalVar[i]   = name of i-th local var    */
    int  *piLocalVarDim;  /* piLocalVarDim[i] = dimension of i-th 
                             local var (=0 for scalar var, <0 for const)  */
    struct function_ *next;  
} function_t;

typedef struct context_{
  function_t *pFunction;  /* ptr to function_t struct of invoked function */
  Instruction_t *pReturn; /* the instruction to continue with after return.*/
  float  **ppfLocalVar;   /* storage block for adr of local variables.
                             Each ref goes into one of these three dests:
                             i)  somewhere into the stack (float arg)
                             ii) into some context's pfAutoVar (auto var)
                             iii) into pfStaticVar
                          */ 
  float  *pfAutoVar;      /* storage block for automatic variables. The 
                             first iNumArgVar elements are reserved for
                             referencing argument parameters. The remaining
                             space is initialized with a copy of the
                             function_t records's pfAutoVar block and
                             provides storage for the functions automatic
                             variables. */
  struct context_ *prev;  /* ptr to previous context */
} context_t;

extern function_t *gpCurrentFunction;
extern context_t  *gpCurrentContext;

#define INS_CALL    125
#define INS_RETURN  126
#define INS_RETURN0 127

