#ifndef __NST_SCRN_H
#define __NST_SCRN_H

#include "nst_aux5.h"

#define nst_winset0(A) {printf("winset at scrn:%d win=%d\n",__LINE__,A); nst_winset(A); } 

#define NO_BLOCK 1
#define SLIDWIDMIN  10    /* min slider length in chars */
#define DEFAULT_BUTTONWID   10    /* min button width in chars  */
#define DEFAULT_I_NUMWID    10
#define DEFAULT_F_NUMWID    10 
#define LINEGAP   2   /* was 2 */
#define OFFSETFAC 0.4
#define MAXWDWLINES 100
#define MAXBUTTWID 190 /* max button width in chars  */
#define MAXNUMWID 30  /* max number width in chars  */
#define MAXBUTT 3
#define DEFAULT_FORMAT  7.3
#define DEFAULT_WINWID1 200
#define DEFAULT_WINHEIGHT1 200
#define DEFAULT_WINXLL1 50
#define DEFAULT_WINYLL1 750
#define DEFAULT_WINWID2 200
#define DEFAULT_WINHEIGHT2 200
#define DEFAULT_WINXLL2 50
#define DEFAULT_WINYLL2 500
#define DEFAULT_TXTWID 12
#define DEFAULT_NUMWID 8

#define UNSELECTABLE -1
#define SELECTABLE    1

#define JUST_DEACTIVATED -1
#define LONG_DEACTIVATED -2
#define JUST_ACTIVATED 1
#define LONG_ACTIVATED 2

#define UNDERLINE_CH '`'
#define ZOOM_STATUS_TOKEN "@ZOOM"

typedef struct nst_margin{
   int iLine, iLeft;
   struct nst_margin *next;
} nst_margin_t;

#define YTXT(U,I) ((char*) (U)->out[(I)]->p[0]->s)
#define CURSOR_X(U,I) ((int) (U)->out[(I)]->p[1]->s[0])
#define CURSOR_Y(U,I) ((int) (U)->out[(I)]->p[2]->s[0])
#define CMDS_INPUT_WINDOW "%=sSbBdDlvRXCc[]tTmMNPiIjkJfFgGHh<^x|{}'!AaZz?~|>"
#define CMDS_OUTPUT_WINDOW "%=vfPRXHh<t^l!?~>"
#define IS_TEXT_CHAR(I) (strchr("iIjJfFgGtTmM",(I))!=NULL)
#define IS_BUTTON_CHAR(I) (strchr("bBcCdD",(I))!=NULL)
#define IS_SLIDER_CHAR(I) (strchr("sS",(I))!=NULL)
#define IS_I_CHAR(I) (strchr("IiJj",(I))!=NULL)
#define IS_F_CHAR(I) (strchr("fFgG",(I))!=NULL)

extern int NST_I_HIDEWID;  /* additional hidden width of each int-field */
extern int NST_F_HIDEWID;  /* additional hidden width of each float-field */
extern float NST_BLANK_ENTRY;    /* indicator of blank entry field */

/* set/get different warping flags */
#define NST_WARP_WINDOW  1
#define NST_WARP_POINTER 2
#define NST_WARP_DIALOGS 4
#define NST_WARP_MODIFY  8
extern  void nst_add_warping(int);
extern  void nst_del_warping(int);
extern  int  nst_get_warping(int);
extern  void nst_warp_window_to_button(char*,unitptr);
extern  void nst_warp_pointer_to_button(char*,unitptr);
extern  void nst_warp_to_button(char*,unitptr);

typedef int (*input_filter)(char*,char*,float*);

extern unitptr input_window(int n, char *windesc, char *fmt, unittype *dest); 
extern void nst_do_function_key_cmd(int);
extern void nst_do_help_cmd(int,char*);
extern void nst_register_function_key_cmd(int,void(*)(void));
extern void nst_register_help_cmd(int,void(*)(char*));
extern void nst_register_window2(unitptr uWindow);
extern void nst_set_widgets(unitptr);
extern void nst_set_widget_color(int, char*);
extern int nst_height_of_text_widget(int iWhich, unitptr uWindow);
extern int nst_is_input_window_unit(unitptr);
extern int nst_input_window_is_open(unitptr);
extern int nst_is_output_window_unit(unitptr);
extern int nst_is_window_unit(unitptr);
extern int nst_default_filter(char*,char*,float*);
extern void nst_mask_button(char*,unitptr);
extern int  nst_press_labelled_button(char*,unitptr);
extern int  nst_press_marked_button(char,unitptr);
extern void nst_unmask_button(char*,unitptr);
extern int nst_enable_button_reset(void);
extern int nst_disable_button_reset(void);
extern void nst_but3_action_for_input_window(int (*pAction)(void*), void *pData);
extern void nst_move_input_window(int,int,unitptr);
extern input_filter set_nst_inputfilter(int(*)(char*,char*,float*));
extern char *input_wdw_fmt(unitptr);
extern char *nst_replace_spaces(char*,char);
extern unitptr copy_of_input_window(unitptr,int,int,unitptr);
extern void nst_copy_zoom_status(unitptr uTo, unitptr uFrom);
extern void nst_disable_text_input(void);
extern void nst_enable_text_input(void);
extern int  nst_text_input_is_enabled(void);
extern void nst_set_initial_token(char *pcInitialToken,int iEntry);
extern void nst_window_autoexit(int,char*);
extern int last_used_text_field(unitptr u);
extern int set_text_field_focus (unitptr u, int iNum);
extern int nst_fprintf_input_window_var(FILE *fp, unitptr u);
extern char *nst_input_wdw_strcpy(unitptr uWdw, int iField, char *pcStr);
extern char nst_change_button_type(char cNewType, int iField, int iPin, unitptr uWdw);
extern int nst_replace_region(char *pcReplacement, int iField, unitptr uWdw);
extern void nst_set_input_wdw_title(char *pcTitle, unitptr u);

/*

   creates a unit the execution of which opens a window (described by
   windesc) allowing the input of a set of variable values (specified 
   by a format string fmt).
   
   arguments:
   
   windesc : must specify title, position (xll,yll) of lower left 
             corner and width dx and height dy (all numbers refer
             to pixel coordinates) of window.

             Example: windesc = "input 100 500 200 300"
             specifies a window with title "input", lower left
             corner at (100,500) that is 200 pixels wide and
             300 pixels high.

   fmt     : specifies what values can be entered and how this can
             be done. 
             To this end, fmt should contain a number of text lines 
             (separated by line feed characters "\n"), together with 
             control fields (beginning with a "%"-character). Any 
             text that does not belong to a control field is simply
             written to the window. A control field specifies the
             insertion of an input facility at the corresponding text 
             location. Allowed control fields and associated input
             facilities are:

%f1:f2:f3s   Slider with input limits of f1 and f2 and default
             position f2. f1,f2 and f3 must be decimal values
             with an optional sign. If only f1 and f2 are given,
             they are interpreted as lower and upper limits.
             If only f1 is given, a range [0..f1] or [f1..0],
             whichever is not empty, is assumed. If neither f1,
             f2, nor f3 is given, a range of [0,1] is assumed.
             If "s" is replaced by "S", an input is required.

%f1:f2b      Button with two values f1 and f2, the first being
             the default. If only f1 is specified, values of
             0 and f1 are assumed, 0 being the default.
             If neither f1 nor f2 is specified, values 0 and
             1 are assumed, 0 being the default. Button is
             displayed bright (for first value) or dark
             (for second value). If "b" is replaced by "B",
             an input is required.

%R           Return-button. If this button is present, the
             routine will return only after it has been pressed.
             Label is "OK". May occur only once (all subsequent
             occurrences are ignored).

%C           Cancel-button. Restore values present when calling
             this routine and return. May occur only once (all
             subsequent occurrences are ignored).

%ltok:       label previous button, using token tok

%d1.d2f      Print current setting of slider whose
             control field precedes this field (ignored,
             if no such facility is specified). Use field
             width d2 and d1 decimals.

For each line, at most one slider or, alternatively, MAXBUTT buttons are allowed.
Additional specifications are simply ignored.

The unit posesses a single output field with the number of pins
given by the number of sliders and buttons (return and
cancel buttons do not count) defined in format string fmt and recognized
as legal. Sliders/buttons correspond to pin 0,1,2.. in the order of their
occurrence in fmt.

The routine returns, when either a return or cancel-button was pressed,
or all sliders resp. buttons with letters "B", "S" have been used at
least once. If only letters "b", "s" occur, only a single change is
accepted.*/

/***********************************************************************/

extern unittype *output_window(int n, char *windesc, char *fmt, unittype *dest); /*

   creates a unit the execution of which opens a window (described by
   windesc) allowing the output of a set of variable values (specified 
   by a format string fmt).
   
   arguments:

   n       : number of input pins.
   
   windesc : must specify title, position (xll,yll) of lower left 
             corner and width dx and height dy (all numbers refer
             to pixel coordinates) of window.

             Example: windesc = "output 100 500 200 300"
             specifies a window with title "output", lower left
             corner at (100,500) that is 200 pixels wide and
             300 pixels high.

   fmt     : specifies what values can be displayed and how this is
             done. To this end, fmt should contain a number 
             of text lines, together with 
             control fields (beginning with a "%"-character). Any 
             text that does not belong to a control field is simply
             written to the window. A control field specifies the
             display of an output value at the location where the 
             field occurs. 
             Allowed control fields are:

   %R        Insert return-button. If this button is present, the
             routine will stop until this button has been pressed.
             Label is "OK". May be specified only once (all subsequent
             specifications are ignored).

  %d1.d2f    Insert decimal representation of next pin value
             (first value = pin0, second value = pin1 ...)
             Use field width d2 and d1 decimals.

*/

extern int nst_fscanf_input_window(FILE*,char*,unitptr);
extern int nst_fprintf_input_window(FILE*,unitptr);
extern unitptr nst_info_box(char*);
extern unitptr nst_choice_box(char*);
extern void nst_alert_box(char*);

extern unitptr view_text(
   char *pcStr,    /* the text to be viewed. NOTE: the memory
                      pointed to by pcStr may be reallocated by this
                      routine and will be freed when the unit is
                      removed! Thus, don't rely on using .pcStr otherwise! */
   int  iMaxLines, /* max nr of lines to view (0=all) */
   int  iWidth,     /* width of textwdw in chars (0=auto) */
   int  iHeight,    /* height of textwdw in chars */
   char *pcWinDesc, /* window descriptor */
   char *pcPre,    /* pre fmt (may be NULL) */
   char *pcPost,    /* post fmt (may be NULL, but at least
                       "%^" for scrollbar is recommended)
                     */
   unitptr uHost    /* host unit */
);
 
extern unitptr view_file( 
   FILE *fp,    /* ptr into the text file to be viewed. */
   int  iMaxLines, /* max nr of lines to view (0=all) */
   int  iWidth,     /* width of textwdw in chars (0=auto) */
   int  iHeight,    /* height of textwdw in chars */ 
   char *pcWinDesc, /* window descriptor */ 
   char *pcPre,    /* pre fmt (may be NULL) */ 
   char *pcPost,    /* post fmt (may be NULL, but at least 
                       "%^" for scrollbar is recommended) 
                     */ 
   unitptr uHost    /* host unit */ 
); 

extern unitptr print_mat(int,int,char*,char*,unitptr);
extern unitptr printstr_unit(char*,int,int,int,int,unitptr);

extern  unitptr E_input_window(int errline, char *errfile, int n, char*windesc,
   char *fmt, unittype *u);

extern  unitptr E_output_window(int errline, char *errfile, int n, char*windesc,
   char *fmt, unittype *u);


#define input_window(A,B,C,D) E_input_window(__LINE__,__FILE__,(A),(B),(C),(D))
#define output_window(A,B,C,D) E_output_window(__LINE__,__FILE__,(A),(B),(C),(D))


#endif
