/*****************************************************************************
 *
 * nst_solid.h  -  header for 3D-object modeling modules
 *
 * Copyright Helge Ritter, 1994
 *
 * $Source$
 *
 * $Log$
 * Revision 1.8  2005/11/07 07:43:36  rhaschke
 * minor cleanup of window routines
 *
 * Revision 1.7  2001/10/01 13:27:51  rhaschke
 * added configurable mouse and keyboard handling to view3d
 * View3d_AddMouseHandler adds mouse handler (for middle button)
 * View3d_AddKeyHandler adds key handler
 *
 * Revision 1.6  1997/10/13 16:47:08  helge
 * mostly changes to SRCLINEs in wrappers
 *
 * Revision 1.5  1997/08/23 22:38:13  helge
 * some corrections to lighting stuff; view3d coord frame rendering options
 *
 * Revision 1.4  1997/04/02  14:03:00  helge
 * minor changes
 *
 * Revision 1.3  1997/03/13  10:43:07  jan
 * Obsolete dependencies removed.
 *
 *
 *****************************************************************************/

#ifndef __NST_SOLID_H
#define __NST_SOLID_H

#define DEGTORAD  0.01745329251994330

#define DEFAULT_WINXLL1 50
#define DEFAULT_WINYLL1 750
#define DEFAULT_WINWID2 200
#define DEFAULT_WINHEIGHT2 200
#define DEFAULT_WINXLL2 50
#define DEFAULT_WINYLL2 500
#define DEFAULT_WINWID3 450
#define DEFAULT_WINHEIGHT3 450
#define DEFAULT_WINXLL3 300
#define DEFAULT_WINYLL3 500

#define VIEW3D_BLOCK1 0
#define VIEW3D_BLOCK2 4
#define VIEW3D_BLOCK3 5
#define VIEW3D_NODRAW 3
#define VIEW3D_SLAVE 1
#define VIEW3D_PASSIVE 2
#define VIEW3D_INTERACTIVE 0  /* don't use anymore! */

#define NST_W_FIND_VIEW3D_WDW -76543

#define MAXCHILDS 20
#define NCYL 8


typedef struct { /* structure for 3D-vertex */
   float x;  float y;  float z;
} POINT;

typedef struct {    /* defines a single polygon  */
  int   n_vertices; /* number of vertex points of polygon */
  short planar;     /* flag to optimize processing of planar shapes */
  POINT *vertex;    /* start of array of n_vertices vertex points */
  POINT *normal;    /* start of array of n_vertices normal vectors */
} POLYGON;

typedef struct {    /* defines shape of a polyhedroN */
  int n_polygons;
  POLYGON **face;    /* pointer to first element of an array of n_polygons
                       faces */
} POLYHEDRON;

typedef struct _OBJECT{  /* object */
  void (*show)(struct _OBJECT *); /* drawing routine */
  POLYHEDRON *part;
  struct _OBJECT *next;
} OBJECT;


typedef struct NST_LINK{
   int id;   /* number of link */
   float rotz, roty, rotx; /* rotation relative to parent link ksys */
   float tx,ty,tz; /* translation relative to parent link origin */
   float th0;   /* rotation angle defining zero position of link */
   int nc;      /* number of child links */
   struct NST_LINK *chi[MAXCHILDS]; /* pointer array to child links */
   unitptr unit;  /* pointer to nst-unit representing 3D-shape of link */
} NST_LINK;

void nst_free_POLYHEDRON(POLYHEDRON*);
void nst_free_POLYGON(POLYGON*);
void nst_free_OBJECT(OBJECT*);

/* all sizes in cm */
#define PUMA_h  66.04
#define PUMA_a1 0
#define PUMA_a2 43.18
#define PUMA_a3 -2.032
#define PUMA_a4 0
#define PUMA_a5 0
#define PUMA_a6 0
#define PUMA_d1 0
#define PUMA_d2 14.909
#define PUMA_d3 0
#define PUMA_d4 43.307
#define PUMA_d5 0
#define PUMA_d6 5.625

typedef struct {
   /* dimensions and notation follow the book by Gonzale and Lee,
      but all indexing of joints is shifted to start by zero,
      i.e. a2(book) = a1(here) and so on.   */
   float a1,a2,d1,d3,d5;
   float thmin[6], dth[6];  /* thmin[i] = theta_i at lower end of range */
                            /* dth[i] = range */
   /* the following are additional "shape parameters" that affect
      not the kinematics, but the rendering: */
   float r1,l1,r2,l2,r3,l3;
   float siz1[9],siz2[9],siz3[9];
} puma_geom;

typedef puma_geom * puma_geom_ptr;

/* mouse handler stuff */
typedef int(*BeginFunc_t)(int x, int y, 
						  int width, int height, unitptr uView3d);
typedef void(*DoFunc_t)(int x, int y); 
typedef void(*EndFunc_t)(int x, int y);

void View3d_AddMouseHandler (const char *pcName, 
							 BeginFunc_t, DoFunc_t, EndFunc_t);
void View3d_DelMouseHandler (BeginFunc_t bgnFunc);
/* end mouse handler stuff */

typedef int (*KeyCallBack_t)(char c);
void View3d_AddKeyHandler (KeyCallBack_t cb);
void View3d_DelKeyHandler (KeyCallBack_t cb);

extern  unitptr translate_frame(float dx, float dy, float dz, unitptr
host);

/* interface:
   X_in[3]   offset added to translation vector (dx,dy,dz),
             i.e. object coord sys is translated by
             (dx + X_in[0], dy + X_in[1], dz + X_in[2])
*/

extern  unitptr rotate_frame(float angle, char axis, unitptr host);

/* interface:
   X_in[1]   offset added to rotation angle.
             i.e. object coord sys is rotated by
             angle + X_in[0] about its origin
   rotation angle in tenths of degree.
*/

extern  unitptr E_rotate_frame(int, char*, float, char, unitptr);

extern  unitptr rot_frame(float angle, char axis, unitptr host);

/* interface:
   X_in[1]   offset added to rotation angle.
             i.e. object coord sys is rotated by
             angle + X_in[0] about its origin
   rotation angle in floating point.
*/

unitptr scale_frame(float sx, float sy, float sz, unitptr host);

/* scale object coord sys by factors sx*X_in[0]..sz*X_in[2] */

unitptr use_material(int material, unitptr host);

unitptr set_light(int,float,float,float,float,float,float,
                      float,float,float,char*,unitptr);

unitptr use_light(int light, 
float,float,float,float,float,float,float,float,float,
char*,unitptr host); 

unitptr begin_frame(unitptr host);

/* interface: no pins */

unitptr end_frame(unitptr host);

/* interface: no pins */

unitptr world_frame(unitptr host);

/* clears matrix stack and loads unit matrix on top, 
   i.e. sets object coord system equal to world coord system.
   interface: no pins */

unitptr define_world(unitptr);
unitptr obj_to_world(int,unitptr);
unitptr world_to_obj(int,unitptr);
unitptr world_to_window(int,unitptr);
unitptr window_to_world(int,unitptr);

/*
Y_in[3]    local point in object frame
CTL_in[0]  control input
X_out[3]   same point in world coord
*/

/* object drawing routines:  each of the following units has 
associated with it a 3d-object that is drawn, when the unit
is executed. In all cases, parameter win specifies an RGB-window,
which is used for drawing. The object is drawn in a standard
position and orientation relative to the coordinate system
currently specified by the matrix stack.
*/

unitptr make_cone(float r1, float r2, float h, int n, 
unitptr host);

/* draw cone */

unitptr make_pyramid(float dx1, float dy1, float dx2, float dy2,
float h, unitptr host);

/* draw pyramid */

unitptr make_puma(int,unitptr);

void nst_render_puma_robot(float,float,float,float,float,float,int);

unitptr make_xyzframe(float,unitptr);
unitptr make_arrow(float,float,float,float,float,unitptr);

unitptr make_link0(float geo[], int n, unitptr host);

/* draw link0 */

unitptr make_link1(float geo[], int n, unitptr host);

/* draw link1 */

unitptr make_link2(float geo[], int n, unitptr host);

/* draw link2 */

unitptr make_link3(float geo[], int n, unitptr host);

/* draw link3 */

#define  rotate_unit(A,B,C) E_rotate_unit(__LINE__,__FILE__,A,B,C)

extern  int RGB_3d_window(char *title, int xll, int yll, int dx, int dy);

NST_LINK *allocate_nst_link(int, unitptr);

void make_links(NST_LINK*, unitptr);

/* make link *l and all of its children */

extern  unitptr make_object(char*, unitptr);

extern  unitptr frame3d(char*, unitptr);

extern  int is_view3d_unit(unitptr);
 
extern  int ctrl_view3d(int,unitptr);
extern  int exec_view3d(unitptr);
extern  unitptr view3d(char*, char*, int, unitptr*, unitptr);

extern  int isView3dUnit(unitptr u);

extern  unitptr make_mesh(int, int, int, char*, unitptr);

extern  unitptr make_graph(int, unsigned short *, int, char*, unitptr);

extern  unitptr make_graph1(int nodes, unsigned short (*f)(int,int), int dim, char* fmt, unitptr host);

extern int is_draw_unit(unitptr u);

extern float *nst_parameters_of_material(int);
extern void draw_object(OBJECT *p);

extern OBJECT *mk_xyzframe(float);
extern OBJECT *mk_icosahedron(int,float*);
extern OBJECT *mk_cone(float r1, float r2, float h, int n);
extern OBJECT *mk_prism(float dx1, float dy1, float dx2, float dy2, float dz);
extern OBJECT *mk_arrow(float, float, float, float, float);
extern OBJECT *mk_linktype0(float);
extern OBJECT *mk_linktype1(float r1,float l1,float a1,float b1,
                        float r2,float l2,float a2,float b2,float h, int n);
extern OBJECT *mk_linktype2(float r1,float l1,float a1,float b1,
                        float r2,float l2,float a2,float b2,float h, int n);
extern OBJECT *mk_linktype3(float r1,float l1,float a1,float b1,
                        float r2,float l2,float a2,float b2,float h, int n);

extern void def_default_lighting(void);
extern void use_default_lighting();
extern float *nst_bind_material(int);

#endif
