/*****************************************************************************
 *
 * nst_types.h  -  NST type definitions
 *
 * Copyright Helge Ritter, 1992
 *
 * $Source$
 *
 * $Log$
 * Revision 1.9  2005/07/12 06:39:34  rhaschke
 * added compiler-switch for EFENCE
 *
 * Revision 1.8  2005/06/01 14:46:51  rhaschke
 * minor fixes to allow use of AUXMALLOC
 *
 * Revision 1.7  2002/01/15 09:24:23  helge
 * - added element to vqueue struct for new ringbuffer feature in nst6.c
 *
 * Revision 1.6  2001/09/14 14:35:38  helge
 * further minor changes
 *
 * Revision 1.5  1999/11/17 17:40:07  helge
 * redefinition of NstInt_t from short int to int
 *
 * Revision 1.4  1999/05/31 18:15:07  helge
 * Added inob-field (inob=interface-object) in *unitptr struct to allow some
 * Neo-related extensions. Other changes mainly removal of files from standard
 * libnst.a into dynamic folders (moved into foldersrc, cf. files added there)
 *
 * Revision 1.3  1997/04/02 14:11:11  helge
 * most man pages now in source code; some corrections;
 * for list, check out new log file in main dir
 *
 * Revision 1.2  1997/03/13  10:51:04  jan
 * Dependency cleanup.
 *
 *
 *****************************************************************************/

#ifndef __NST_TYPES_H
#define __NST_TYPES_H
#include <stdio.h>

#define LABEL_LEN 20     /* maximal length of name allowed for a unit */

#ifdef AUXMALLOC
#include "aux_malloc.h" 
/* provide an unsubstituted free function: */
#define free0(p) AUX_MALLOC_DISABLE free(p); AUX_MALLOC_ENABLE
#else
#define free0 free
#endif

#ifdef EFENCE
#include <efence/efence.h>
#endif

typedef unsigned char byte;

typedef int NstInt_t;   
/* HR: CHANGED 8/8/99: now use ints for int-fields for better compatibility
   (were short ints before, but only used in few places) 
*/

typedef void (*conv_funct_t)(int,void*,void*);

typedef struct def_pin_extension{  
  /* struct for extension of pin to vector or port type: */
  void *host;  /* ptr to host port */
  int  iDim;       /* nr of vector elements */
  char native;     /* native type of pin, one of ifcbt */
  /* ptrs to converted versions of data vector */
  float *pf; /* result of conversion native->float          */
  NstInt_t  *pi; /* result of conversion native->int            */
  char  *pc; /* result of conversion native->unsigned char  */
  void  *ps; /* ... */ 
  /* ptrs to conversion functions for conversions native->xxx: */
  conv_funct_t pFf,  /* converts native->float         */
    pFi,  /* converts native->int           */
    pFc;  /* converts native->unsigned char */
} pin_extension;


typedef float VARTYPE;

typedef struct defpin{
  /* struct for basic pin type: */
  struct defpin *r;  /* ptr to first reader of this pin; NULL if none */
  struct defpin *w;  /* if t & NST_PINSTATUS == 0:
			ptr to writer of this pin;
			NULL if none;
			VPIN, if pin this pin is bound to a C-variable
			if t & NST_PINSTATUS == 1:
			ptr to next pin of this pin's readergroup.
			[the masking with NST_PINSTATUS is new]
			*/
  char t;             /* flag [see above] */
  VARTYPE *s;            /* ptr to value of this pin */
  pin_extension *ext;
} pin;

typedef pin   NstPin;

typedef struct def_rectangle_t{
  int xll,yll,w,h;  /* lower left corner
		       and width and height of rectangle area */
} rectangle_t;

typedef struct def_field{
  pin   **p;             /* pointer to array of pins */
  int npins;             /* number of pins */
  int dim;               /* dim of field (negative, if packed) */
  int id;                /* integer id of field */
  unsigned int property; /* bit field for storing various properties */
  int dx;                /* valid only for packed field: width of pixel rectangle */
  rectangle_t r;         /* valid only for packed field: restriction area info */
  void *host;            /* pointer to host unit */
} field;

typedef struct def_vqueue_el{  /* implements variable queue for units */
  void *el; int numel;   
  /* CHANGED! [don't worry, only used in 3 very select places, see nst6.h] */
  struct def_vqueue_el *next,*prev;
  /* !!! linear/circular list in forward/backward order !!!
     first element -> prev = last element of forward list, i.e.
     but first element -> prev->next = NULL !!!
  */
} vqueue_el;

typedef struct def_unitlistel{
   /* struct def_unittype */ void *u;
   struct def_unitlistel  *prev, *next;
} unitlistel;

typedef struct def_unittype{
   struct def_unittype *myself;  /* must be == u (used for error checking) */
   field **in;
   field **out;
   int ninputs, noutputs;
   struct def_unittype
   *host,          /* pointer to this unit's host unit */
   *first_subunit, /* ptr to first subunit, NULL if none */
   *last_subunit,  /* ptr to last subunit, NULL if none */
   *prev, *next;   /* ptr to predecessor/successor of this unit
                     in the list of subunits of host, NULL if none */
   
   float *ctlvalptr;  /* ptr to a pin value; this pin then determines
                         whether unit is executed or not [=value 0] */ 

   int operands;      /* nr of operands, if operator unit */
   int first_operand; /* offset to first operand */ 
   int (*exec)(struct def_unittype*);
   int (*remove)(struct def_unittype*);
   int (*adapt)(struct def_unittype*);
   int (*ctrl)(int, struct def_unittype*);
   int (*load)(FILE*, struct def_unittype*);
   int (*save)(FILE*, struct def_unittype*);

   void* var; /* pointer to array of local variables referenced by the above
	      routines. Memory must be allocated by routine init, and the
	      pointer var must be directed to its first element.  */

   char name[LABEL_LEN]; /* print name of unit. Must not contain spaces. */
   unsigned 
   long property; /* a set of bit flags indicating various properties */
 		  /* the following are pointers to the basic
		     methods associated with each unit :           */
   int dbx;       /* field for debugging info */

   /* the following fields specify further local data of a unit: */

   struct def_vqueue_el  *qp;
	   /* queue pointer pointing to currently used local variable */

   int nvar;  /* number of local variables of type value used (i.e. 1+number
	      of elements of type value following first element var[0])  */
   void *inob; /* ptr to optional interface object (HR 5/98) */
} unittype, *unitptr;

typedef int (*exec_function_t)(unitptr);


#endif
