/*
   Compile this file into demolib2.so. Then a
   prog_unit can use the functions below after a

   #import "demolib2"

   directive.
*/
    
#include <string.h>
#include <stdio.h>

char *FUNCTIONS_demolib2[] = {
   /* define a prefix: */
   "__:",
   /* two exported constants: */
   "float MyPI", 
   "int   My4711", 
   /* three exported functions: */
   "void  foo(int, float*)",
   "float bar(void)",
   "int   bcopy(char*, byte*)",
   /* overloading: use a separate prefix 
      for each function variant: */
   "_1_:", "fun(int)",
   "_2_:", "fun(float)",
   "_3_:", "fun(int,float)",
   /* a vararg function: */
   "__:",  "myfun(float,int*...)",
   NULL  /* end mark */
};

/* the implementation: parameter list is always

 (int *piDim, void **ppvArg)          for non-vararg functions
 (int iNumDim, *piDim, void **ppvArg) for vararg functions

 with  piDim[i]  = dimension of i-th argument (0 = scalar)
       ppvArg[i] = address of first el of i-th argument
*/

float __MyPI   = 3.14;
int   __My4711 = 4711;

void __foo(int *piDim, void **ppvArg) {
   int i;
   int   *piArg0 = (int*)   ppvArg[0];
   float *pfArg1 = (float*) ppvArg[1];
   printf("calling foo: first arg is    = %d\n",*piArg0);
   printf("             2nd arg has dim = %d. \nElements are:\n",piDim[1]);
   for (i=0; i<piDim[1]; i++) {
      printf("el[%d] = %g\n",i,pfArg1[i]);
   }
}

float __bar(int *piDim, void **ppvArg) {
/* a parameterless function ignores its arguments */
   printf("calling bar()\n");
   return 12345.;
}

int __bcopy(int *piDim, void **ppvArg) {
/* copies chars of first arg into 2nd arg 
   (truncates according to available space) 
*/
   char   *pcArg0  = (char*) ppvArg[0];
   char   *pcArg1  = (char*) ppvArg[1];
   int iAvailBytes = piDim[1]-1;
   if (iAvailBytes <= 0) return 0; 
   if (pcArg0 && pcArg1) strncpy(pcArg1,pcArg0,iAvailBytes);
   return iAvailBytes;
}

/* the overloaded functions fun(..): */

float _1_fun(int *piDim, void **ppvArg) {
   int *piArg0 = (int*) ppvArg[0];
   printf("calling fun(int = %d)\n",*piArg0);
   return 0;
}

float _2_fun(int *piDim, void **ppvArg) {
   float *pfArg0 = (float*) ppvArg[0];
   printf("calling fun(float = %g)\n",*pfArg0);
   return 0;
}

float _3_fun(int *piDim, void **ppvArg) {
   int   *piArg0 = (int*) ppvArg[0];
   float *pfArg1 = (float*) ppvArg[1];
   printf("calling fun(int = %d float = %g)\n",*piArg0,*pfArg1);
   return 0;
}

/* the vararg function myfun(): */

float __myfun(int iNumArgs, int *piDim, void **ppvArg) {
   float   *pfArg0 = (float*) ppvArg[0];
   int i,j,iDim, *piArg;
   printf("calling myfun: first arg is    = %g,\n",*pfArg0);
   if (iNumArgs > 1) {
      printf("remaining args are: \n");
      for (i=1; i<iNumArgs; i++) {
         /* for (type ...) all varargs are of the type of the
            last non-vararg ("int" in the present case) */ 
         piArg = (int*) ppvArg[i];
         iDim  = piDim[i];
         printf("   arg#%d = ",i);
         if (iDim > 0) for (j=0; j<iDim; j++) printf("%d ",piArg[j]);
         else printf(" NULL");
         printf("\n");
      }
   }
   return 0;
}
