/**********************************************************************

  Subclass "picture1":

  extends class "picture" from demolib5b.c by new element functions:

  add(line_t ...)   add zero or more lines to picture 
                    [illustrates overloading of existing add() 
                     function of base class] 

  translate(float,float)  translate all lines and points by 2d vector
                    [illustrates addition of a new function]

  list(void)        replaces existing list() function of base class
                    with a new list function that also lists both the
                    added points *and* the newly added lines.
                    [illustrates overwriting of an existing base 
                     class function]

  Additionally, the package defines a load/save function pair that
  deals with the newly introduced data and thereby extends the
  capabilities of the existing load/save functions of the base
  class appropriately.

***********************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "demolib5a.h"   /* headers for point and line class   */
#include "demolib5b.h"   /* headers for base class picture     */
#include "demolib5c.h"   /* headers for derived class picture1 */

/************************* in demolib5c.h: ****************************

typedef struct picture1_{ 
   picture_t base;             // the base class data 
   // new data elements to extend base class: 
   int iNumLines, iMaxLines;   // current and max nr of lines  
   line_t **ppLine;            // pointer array to iNumLines  
                               // line_t instances 
} picture1_t;  // the C-struct to hold an instance of class "picture1" 

***********************************************************************/

   char *picture1_c[] = {

      /* define derived class: */
      "EXTENDS pic.picture FROM demolib5b", 

      "C_:",                    /* name prefix       */
      "picture1(void)",         /* constructor       */
      "~picture1()",            /* destructor        */
      "add(class geo.line ...)",  /* add some lines    */
      "void translate(float,float)", /* translate all objects */
      "void list(void)",             /* new list function */
      "SAVE_picture1()",        /* save function     */
      "LOAD_picture1()",        /* load function     */
      "RESTORE_picture1()",     /* restore function  */

      /* allow prog_unit check of optional version info: */

      PICTURE1_VERSION, PICTURE_VERSION, LINE_VERSION,

      /* each token holds a string "classname = version". Tokens of
         this structure are "understood" by the prog_unit and will
         be used for version checking (a full consistency check 
         requires one token for the defined class [here: "picture1"] 
         and for each class used as an argument type [here: "line"] 
         or as a base class [here: "picture"]). The "classname=version" 
         token for each class might e.g. be #defined in its header file.
      */

   NULL};

   char **CLASSES[] = {picture1_c, NULL};

#define RESIZE(Data,MaxSize,Type) \
   /* this auxiliary macro changes its arguments!! */ \
   Data = realloc(Data,(MaxSize=2*MaxSize)*sizeof(Type))

/*************************************************************************
  class constructor:  
*************************************************************************/

picture1_t *C_picture1(void) {

   /* first create instance of base class: */
   picture_t  *pData  = B_picture(NULL,NULL);
   /* then resize to derived subclass: */
   picture1_t *pData1 = realloc(pData,sizeof(picture1_t));

   printf("calling constructor picture1(void)\n");

   /* now the initial portion of pData1 can be used as
      instance of the base type, the rest holds any new
      stuff. Initialize the new stuff, as required:
   */
   pData1->iNumLines = 0;
   /* set aside initial memory for lines: */
   pData1->iMaxLines = 10;
   pData1->ppLine   = calloc(10,sizeof(pData1->iMaxLines));

   return pData1;
}

/*************************************************************************
  class destructor:
*************************************************************************/

void C_FREE_picture1(picture1_t *pData) {

   /* implementation must permit NULL argument! */

   if (pData) { int k;
      /* clean up everything that does not belong
         to base class (in this example, nothing to do)
      */
      for (k=0; k<pData->iNumLines; k++) free(pData->ppLine[k]);
      free(pData->ppLine);

      /* then call base class destructor: */
      B_FREE_picture((picture_t*) pData);
      /* the final free will have freed the *entire*, 
         larger picture1_t - block! 
      */
   }
}

/*************************************************************************
  class load/save functions: need only deal with new data!
*************************************************************************/

void C_SAVE_picture1(FILE *fp, picture1_t *pData) {
   
   /* details uninteresting, main message is that only
      additional data elements (i.e., lines) need to be saved! */

   int i;  line_t *pLine;
   fprintf(fp,"%d\n",pData->iNumLines);
   for (i=0; i<pData->iNumLines; i++) {
      pLine = pData->ppLine[i];
      fprintf(fp,"%g %g %g %g\n",pLine->x1,pLine->y1,pLine->x2,pLine->y2);
   }
}

void C_LOAD_picture1(FILE *fp, picture1_t *pData) {

   /* details uninteresting, main message is that only
      additional data elements (i.e., lines) need to be loaded! */

   int i, iNumLines=0;  line_t *pLine;

   fscanf(fp,"%d",&iNumLines);

   /* ensure sufficient space: */
   while (iNumLines >= pData->iMaxLines) {
      RESIZE(pData->ppLine, pData->iMaxLines, line_t);
   }

   /* load the iNumLines from file: */
   for (i=0; i<iNumLines; i++) {
      if (i < pData->iNumLines) {
         pLine = pData->ppLine[i];
      } else {
         pLine = pData->ppLine[i] = malloc(sizeof(line_t));
      }
      fscanf(fp,"%g %g %g %g",&(pLine->x1), &(pLine->y1), 
                        &(pLine->x2), &(pLine->y2));
      /* real code would add error checking! */
   }
   /* free any unused points: */
   for (; i<pData->iNumLines; i++) free(pData->ppLine[i]);
   pData->iNumLines = iNumLines;

}

void C_RESTORE_picture1(FILE *fp, picture1_t **ppData) {

   /* the RESTORE method is required to make an instance, if 
      (and *only if*) the passed handle adr ppData contains 
      a NULL pointer for the class handle *ppData: 
   */

   if (!(*ppData)) *ppData = C_picture1();

   /* the rest must be a normal load: */

   C_LOAD_picture1(fp, *ppData); 
   
}

/*************************************************************************
  overload existing add() method of base class:
*************************************************************************/

void C_add(int iNumArg, int *piDim, void **ppvArg, picture1_t *pData) {

   int i,j; line_t *pArg;

   printf("called drawing.add(class line ... )\narguments are:\n");
   for (i=0; i<iNumArg; i++) {
      pArg = (line_t*) ppvArg[i];
      printf("line #%d:  (%g %g) -> (%g %g)\n",
              i, pArg->x1, pArg->y1, pArg->x2, pArg->y2);
   }

   /* code to add lines to picture pData: */
   while (iNumArg + pData->iNumLines >= pData->iMaxLines) {
       RESIZE(pData->ppLine,pData->iMaxLines,float*);
   }
   for (i=pData->iNumLines, j=0; i<pData->iNumLines+iNumArg; i++,j++) {
      pData->ppLine[i] = memcpy(malloc(sizeof(line_t)),ppvArg[j],sizeof(line_t));
   }
   pData->iNumLines = i;
}

/*************************************************************************
  provide new translate() method:
*************************************************************************/

void C_translate(int *piDim, void **ppvArg, picture1_t *pData) {
  
/* example of a new function that needs action on both the new
   (=lines) and the old (=points) data elements in the instance
*/
 
   float x = *((float*) ppvArg[0]);
   float y = *((float*) ppvArg[1]);

   /* translate all lines by (x,y): */
   picture_t *pBaseData = (picture_t*) pData;
   int k;
   for (k=0; k<pData->iNumLines; k++) {
      pData->ppLine[k]->x1 += x;
      pData->ppLine[k]->x2 += x;
      pData->ppLine[k]->y1 += y;
      pData->ppLine[k]->y2 += y;
   }

   /* now work on variables of base class to translate all points: */
   for (k=0; k<pBaseData->iNumPoints; k++) {
      pBaseData->ppPoint[k]->x += x;
      pBaseData->ppPoint[k]->y += y;
   }
}

/*************************************************************************

  Replace list() method of base class by new version that can also
  handle the newly introduced line items [we still call the old function 
  to do the listing for point items].

  NOTE: the replacement occurs in the name space of the prog_unit!
        the C-functions at the implementation level all coexist
        simultaneously (distinguishable by their different prefixes)  

*************************************************************************/

void C_list(int *piDim, void **ppvArgs, picture1_t *pData) {

   int i;

   /* use base function to print the base elements: 
      [if the parameters of B_list() would differ, we would have
       to build corresponding piDim and ppvArg arguments!]
   */
   B_list(piDim, ppvArgs, (picture_t*) pData);

   /* now append listing of new data elements: */

   printf("Picture %p has %d lines:\n", pData, pData->iNumLines);
   for (i=0; i<pData->iNumLines; i++) {
      line_t *pLine = pData->ppLine[i];
      printf("line#%d = (%g %g) -> (%g %g)\n",i,
              pLine->x1,pLine->y1,pLine->x2,pLine->y2);
   }
}

